﻿using System;
using System.ServiceModel;
using System.Collections.Generic;
using Firma.Anwendung.BusinessEntities;

namespace Firma.Anwendung.Services
{
    [ServiceContract(Namespace = "Firma.Anwendung.Services", Name = "ICustomerService", SessionMode = SessionMode.NotAllowed)]
    public interface ICustomerService
    {
        #region Service Specific
        /// <summary>
        /// Get information (e.g. name, version) about the service
        /// </summary>
        /// <returns>Information about the service</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        ServiceInfo GetServiceInfo();
        #endregion

        #region Customer
        /// <summary>
        /// Crestes a new static customer object
        /// </summary>
        /// <param name="customer">The customer object to be created</param>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        void CreateCustomer(Customer customer);

        /// <summary>
        /// Updates the static customer object
        /// </summary>
        /// <param name="customer">The customer object to be updated</param>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        void UpdateCustomer(Customer new_customer, Customer old_customer);

        /// <summary>
        /// Deletes the static customer object
        /// </summary>
        /// <param name="customerId">The application id of the customer</param>
        [OperationContract]
        [FaultContract(typeof(ServiceValidationFault))]
        [FaultContract(typeof(ServiceFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        void DeleteCustomer(String customerId);

        /// <summary>
        /// Returns the static customer object or null
        /// </summary>
        /// <param name="customerId">The application id of the customer</param>
        /// <returns>Customer object or null if no customer object exist</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Customer GetCustomerById(String customerId);

        /// <summary>
        /// Returns an array of  customer objects
        /// </summary>
        /// <returns>Array of customer objects or empty array if no customer objects are defined</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Customer [] GetCustomerArray();

        /// <summary>
        /// Returns an array of  customer objects with a name
        /// </summary>
        /// <returns>Array of customer objects or empty array if no customer objects are defined</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Customer[] GetCustomerArrayForName(String companyName);
        #endregion

    }
}
