﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using System.Reflection;
using Firma.Anwendung.Utility;

#pragma warning disable 168


namespace Firma.Anwendung.Service
{
    /// <summary>
    /// Unser Application Service
    /// </summary>
    public partial class ETAppService : ServiceBase
    {
        private readonly LogUtility m_oLogUtility = new LogUtility(typeof(ETAppService));
        private readonly List<ETServiceModule> m_oETServiceModuleList = new List<ETServiceModule>();
        private readonly ManualResetEvent m_oKillEvent = new ManualResetEvent(false);
        private const int THREADSTOPTIMEOUT = 5000;
        private const int REQUESTADDITIONALTIME = 5000;

        /// <summary>
        /// Unser Application Service Constructor
        /// </summary>
        public ETAppService()
        {
            InitializeComponent();
        }
                /// <summary>
        /// ReadSettings
        /// </summary>
        /// <returns></returns>
        public bool ReadSettings()
        {
            bool fResult = false;
            string strConfig = ".config";

            try
            {
                Module oModule = Assembly.GetExecutingAssembly().GetModules()[0];
                if (oModule != null)
                {
                    string strConfigFile = oModule.FullyQualifiedName + strConfig;
                    if (File.Exists(strConfigFile))
                    {
                        // Konfiguration lesen
                        string strModulesEntry = Properties.Settings.Default.Modules;
                        if (!string.IsNullOrEmpty(strModulesEntry))
                        {
                            string[] strModules = strModulesEntry.Split('|');
                            if (strModules.Length > 0)
                            {
                                string strPath = Path.GetDirectoryName(oModule.FullyQualifiedName);
                                foreach (string strModule in strModules)
                                {
                                    string strAssembly = Path.GetFullPath(strPath + "\\" + strModule.Trim(' '));
                                    if (File.Exists(strAssembly))
                                    {
                                        ETServiceModule oETServiceModuleList = new ETServiceModule(strAssembly, m_oKillEvent, EventLog);
                                        m_oETServiceModuleList.Add(oETServiceModuleList);
                                    }
                                    else
                                    {
                                        m_oLogUtility.TraceLog(LogLevel.Error, "Not found module: " + strAssembly);
                                        EventLog.WriteEntry("Not found module: " + strAssembly, EventLogEntryType.Error);
                                    }
                                }

                                fResult = m_oETServiceModuleList.Count > 0 ? true : false;
                            }
                        }
                        else
                        {
                            m_oLogUtility.TraceLog(LogLevel.Error, "Not found module entry: " + strModulesEntry);
                        }
                    }
                    else
                    {
                        m_oLogUtility.TraceLog(LogLevel.Error, "Not Found: " + strConfigFile);
                    }
                }
                else
                {
                    m_oLogUtility.TraceLog(LogLevel.Error, "Not Found: GetExecutingAssembly");
                }
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, "ReadSettings", oException);
            }

            return fResult;
        }

        /// <summary>
        /// StartModules
        /// </summary>
        /// <returns></returns>
        private bool StartModules()
        {
            bool fResult = true;

            try
            {
                foreach (ETServiceModule oETServiceModule in m_oETServiceModuleList)
                {
                    RequestAdditionalTime(REQUESTADDITIONALTIME);

                    if (!oETServiceModule.StartModule())
                    {
                        m_oLogUtility.TraceLog(LogLevel.Error, "Not Start Module: " + oETServiceModule.AssemblyName);
                        fResult = false;
                        break;
                    }
                }
            }
            catch (Exception oException)
            {
                fResult = false;
                m_oLogUtility.TraceException(LogLevel.Error, "StartModules", oException);
            }

            return fResult;
        }

        /// <summary>
        /// OnStart
        /// </summary>
        /// <param name="args"></param>
        protected override void OnStart(string[] args)
        {
            m_oLogUtility.TraceLog(LogLevel.Info, "OnStart");
            System.Threading.Thread.Sleep(15000);

            try
            {
                m_oLogUtility.TraceLog(LogLevel.Debug, "Product Version: " + Process.GetCurrentProcess().MainModule.FileVersionInfo.ProductVersion);
                foreach (ProcessModule oProcessModule in Process.GetCurrentProcess().Modules)
                {
                    m_oLogUtility.TraceLog(LogLevel.Debug, "Module: " + oProcessModule.FileName + " - Version: " + oProcessModule.FileVersionInfo.ProductVersion);
                }

                if ((!ReadSettings()) || (!StartModules()))
                {
                    OnStop();
                    Environment.Exit(999);
                }
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, "OnStart", oException);
                OnStop();
                Environment.Exit(999);
            }
        }

        /// <summary>
        /// OnStop
        /// </summary>
        protected override void OnStop()
        {
            try
            {
                m_oLogUtility.TraceLog(LogLevel.Info, "OnStop");
                
                m_oKillEvent.Set();

                m_oETServiceModuleList.Reverse();
                foreach (ETServiceModule oETServiceModule in m_oETServiceModuleList)
                {
                    try
                    {
                        if (oETServiceModule.WorkerThread != null)
                        {
                            RequestAdditionalTime(REQUESTADDITIONALTIME);

                            if (!oETServiceModule.WorkerThread.Join(THREADSTOPTIMEOUT))
                            {
                                oETServiceModule.WorkerThread.Abort();
                                m_oLogUtility.TraceLog(LogLevel.Warning, "OnStop - Abort - Module: " + oETServiceModule.AssemblyName);
                            }
                            else
                            {
                                m_oLogUtility.TraceLog(LogLevel.Info, "OnStop Module: " + oETServiceModule.AssemblyName);
                            }
                        }
                    }
                    catch(Exception oException)
                    {
                        m_oLogUtility.TraceException(LogLevel.Error, "OnStop", oException);        
                    }
                }
            }
            catch (Exception oException)
            {
            m_oLogUtility.TraceException(LogLevel.Error, "OnStop", oException);
            }
        }
    }
}

