﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using System.Security.Principal;
using System.Text;
using System.Threading;
// using System.Web.Security;
using Firma.Anwendung.Utility;

//The variable 'var' is assigned but its value is never used CS0168
#pragma warning disable 168


namespace Firma.Anwendung.Service
{
    /// <summary>
    /// ETServiceModule
    /// </summary>
    public class ETServiceModule
    {
        /// <summary>
        /// LogUtility
        /// </summary>
        private readonly LogUtility m_oLogUtility = new LogUtility(typeof(ETServiceModule));

        /// <summary>
        /// AssemblyName
        /// </summary>
        private readonly string m_strAssemblyName = string.Empty;

        /// <summary>
        /// WorkerThread
        /// </summary>
        private Thread m_oWorkerThread = null;

        /// <summary>
        /// KillEvent for Thread
        /// </summary>
        private readonly ManualResetEvent m_oKillEvent = null;

        /// <summary>
        /// EventLog
        /// </summary>
        private readonly EventLog m_oEventLog = null;

        /// <summary>
        /// Thread TIMEOUT
        /// </summary>
        private const int TIMEOUT = 5000;

        /// <summary>
        /// Init Success
        /// </summary>
        private bool m_fInitSuccess = false;

        /// <summary>
        /// ETServiceModule
        /// </summary>
        /// <param name="strAssemblyName">Path Assembly</param>
        /// <param name="oKillEvent">Kill Event from Parent (Windows Service)</param>
        public ETServiceModule(string strAssemblyName, ManualResetEvent oKillEvent, EventLog oEventLog)
        {
            m_strAssemblyName = strAssemblyName;
            m_oKillEvent = oKillEvent;
            m_oEventLog = oEventLog;
        }

        /// <summary>
        /// AssemblyName
        /// </summary>
        public string AssemblyName
        {
            get { return m_strAssemblyName; }
        }

        /// <summary>
        /// WorkerThread
        /// </summary>
        public Thread WorkerThread
        {
            get { return m_oWorkerThread; }
        }

        /// <summary>
        /// ThreadData
        /// </summary>
        private class ThreadData
        {
            /// <summary>
            /// AutoResetEvent
            /// </summary>
            private readonly AutoResetEvent m_oInitEvent = null;

            /// <summary>
            /// EventLog
            /// </summary>
            private readonly EventLog m_oEventLog = null;

            /// <summary>
            /// ThreadData
            /// </summary>
            /// <param name="oInitEvent"></param>
            /// <param name="oEventLog"></param>
            public ThreadData(AutoResetEvent oInitEvent, EventLog oEventLog)
            {
                m_oInitEvent = oInitEvent;
                m_oEventLog = oEventLog;
            }

            /// <summary>
            /// InitEvent
            /// </summary>
            public AutoResetEvent InitEvent
            {
                get { return m_oInitEvent; }
            }

            /// <summary>
            /// EventLog
            /// </summary>
            public EventLog EventLog
            {
                get { return m_oEventLog; }
            }
        }

        /// <summary>
        /// MainWorkerThread
        /// </summary>
        /// <param name="oData"></param>
        private void MainWorkerThread(object oData)
        {
            ThreadData oThreadData = (ThreadData)oData;

            try
            {
                Assembly oAssembly = Assembly.LoadFrom(m_strAssemblyName);
                if (oAssembly != null)
                {
                    object[] oAttributs = oAssembly.GetCustomAttributes(typeof(ModuleAttribute), false);
                    if ((oAttributs != null) && (oAttributs.Length == 1) && (oAttributs[0] != null))
                    {
                        object oModule = oAssembly.CreateInstance(((ModuleAttribute)oAttributs[0]).TypeName);
                        if (oModule != null)
                        {
                            //// Set Principal
                            //AppDomain.CurrentDomain.SetPrincipalPolicy(PrincipalPolicy.WindowsPrincipal);
                            m_oLogUtility.TraceLog(LogLevel.Info, "User: " + WindowsIdentity.GetCurrent().Name);
                            //GenericIdentity oGenericIdentity = new GenericIdentity(WindowsIdentity.GetCurrent().Name);
                            //RolePrincipal oRolePrincipal = new RolePrincipal(oGenericIdentity);
                            //Thread.CurrentPrincipal = oRolePrincipal;

                            if (((BaseModule)oModule).Init())
                            {
                                m_oLogUtility.TraceLog(LogLevel.Info, "Start module: " + m_strAssemblyName);
                                oThreadData.EventLog.WriteEntry("Start module: " + m_strAssemblyName, EventLogEntryType.Information);
                                m_fInitSuccess = ((BaseModule)oModule).Start();
                                if (m_fInitSuccess)
                                {
                                    // Init Event for Windows Service
                                    oThreadData.InitEvent.Set();

                                    // Wait Stop for Windows Service
                                    m_oKillEvent.WaitOne();

                                    ((BaseModule)oModule).Stop();

                                    if ((((BaseModule)oModule).WorkerThread != null) && (((BaseModule)oModule).WorkerThread.Length > 0))
                                    {
                                        for (int i = 0; i < ((BaseModule)oModule).WorkerThread.Length; i++)
                                        {
                                            if (((BaseModule)oModule).WorkerThread[i] != null)
                                            {
                                                if (!((BaseModule)oModule).WorkerThread[i].Join(TIMEOUT))
                                                {
                                                    ((BaseModule)oModule).WorkerThread[i].Abort();
                                                    m_oLogUtility.TraceLog(LogLevel.Warning, "OnStop - Abort - Module: " + m_strAssemblyName);
                                                    oThreadData.EventLog.WriteEntry("OnStop - Abort - Module: " + m_strAssemblyName, EventLogEntryType.Warning);
                                                }
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    m_oLogUtility.TraceLog(LogLevel.Error, "Not started module: " + m_strAssemblyName);
                                    oThreadData.EventLog.WriteEntry("Not started module: " + m_strAssemblyName, EventLogEntryType.Error);
                                }
                            }
                            else
                            {
                                m_oLogUtility.TraceLog(LogLevel.Error, "Not init module: " + m_strAssemblyName);
                                oThreadData.EventLog.WriteEntry("Not init module: " + m_strAssemblyName, EventLogEntryType.Error);
                            }
                        }
                        else
                        {
                            m_oLogUtility.TraceLog(LogLevel.Error, "Not create instance module: " + m_strAssemblyName);
                            oThreadData.EventLog.WriteEntry("Not create instance module: " + m_strAssemblyName, EventLogEntryType.Error);
                        }
                    }
                }
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, "MainWorkThread - Module: " + m_strAssemblyName, oException);
            }
            finally
            {
                if (!m_fInitSuccess)
                {
                    oThreadData.InitEvent.Set();
                }
            }
        }

        /// <summary>
        /// StartModule
        /// </summary>
        /// <returns></returns>
        public bool StartModule()
        {
            bool fResult = false;

            try
            {
                m_oWorkerThread = new Thread(MainWorkerThread);
                AutoResetEvent oInitEvent = new AutoResetEvent(false);
                ThreadData oThreadData = new ThreadData(oInitEvent, m_oEventLog);
                m_oWorkerThread.Start(oThreadData);
                fResult = oInitEvent.WaitOne() ? m_fInitSuccess : false;
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, "StartModule", oException);
            }

            return fResult;
        }
    }
}

