﻿using System;
using System.IO;
using System.Reflection;
using System.Security.Principal;
using System.Threading;
using System.Web;
using System.Web.Security;
// using Firma.Anwendung.BusinessEntities;
using Firma.Anwendung.Services;
using Firma.Anwendung.Utility;

namespace Firma.Anwendung.Host.IISHosting
{
    /// <summary>
    /// IIS Hosting
    /// </summary>
    public class Global : HttpApplication
    {
        private static LogUtility m_oLogUtility = new LogUtility("IISHosting");

        /// <summary>
        /// CheckServiceConfiguration
        /// </summary>
        /// <returns>successful</returns>
        private bool CheckServiceConfiguration(HttpContext oHttpContext, HttpServerUtility oHttpServerUtility)
        {
            bool fResult = true;

            try
            {
                //GenericIdentity oGenericIdentity = new GenericIdentity(WindowsIdentity.GetCurrent().Name);
                //RolePrincipal oRolePrincipal = new RolePrincipal(oGenericIdentity);
                //Thread.CurrentPrincipal = oRolePrincipal;

                oHttpContext.User = Thread.CurrentPrincipal;

                m_oLogUtility.TraceLog(LogLevel.Info, "Check Service Configuration");
                string strPath = oHttpServerUtility.MapPath("/NWServicesIIS");
                if (Directory.Exists(strPath))
                {
                    DirectoryInfo oDirectoryInfo = new DirectoryInfo(strPath);
                    FileInfo[] oFileInfos = oDirectoryInfo.GetFiles("*.svc");
                    if ((oFileInfos != null) && (oFileInfos.Length > 0))
                    {

                        foreach (FileInfo oFileInfo in oFileInfos)
                        {
                            string strFileName = strPath + "\\bin\\" + oFileInfo.Name.Replace(".svc", ".dll");
                            if (File.Exists(strFileName))
                            {
                                Assembly oAssembly = Assembly.LoadFrom(strFileName);
                                if (oAssembly != null)
                                {
                                    Type oType = oAssembly.GetType("Firma.Anwendung.Services." + oFileInfo.Name.Replace(".svc", ""));
                                    if (oType != null)
                                    {
                                        Object oObject = Activator.CreateInstance(oType);
                                        if (oObject != null)
                                        {
                                            ETBaseService oETBaseService = oObject as ETBaseService;
                                            if (oETBaseService != null)
                                            {
                                                IETBaseService oIETBaseService = oObject as IETBaseService;
                                                if (oIETBaseService != null)
                                                {
                                                    oIETBaseService.CheckServiceConfiguration();
                                                    m_oLogUtility.TraceLog(LogLevel.Info, "Read Service Config: " + oFileInfo.Name.Replace(".svc", ".dll"));
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        fResult = true;
                    }
                }
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, "Exception CheckServiceConfiguration: ", oException);
            }

            return fResult;
        }

        /// <summary>
        /// Application_Start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Application_Start(object sender, EventArgs e)
        {
            m_oLogUtility.TraceLog(LogLevel.Info, "IISHosting Application Start");

            CheckServiceConfiguration(Context, Server);
        }

        /// <summary>
        /// Application_End
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Application_End(object sender, EventArgs e)
        {
            m_oLogUtility.TraceLog(LogLevel.Info, "IISHosting Application End");
        }
    }
}