﻿using System;
using System.ServiceModel;
using System.Collections.Generic;
using Firma.Anwendung.BusinessEntities;

namespace Firma.Anwendung.Services
{
    [ServiceContract(Namespace = "Firma.Anwendung.Services", Name = "IProductService", SessionMode = SessionMode.NotAllowed)]
    public interface IProductService
    {
        #region Service Specific
        /// <summary>
        /// Get information (e.g. name, version) about the service
        /// </summary>
        /// <returns>Information about the service</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        ServiceInfo GetServiceInfo();
        #endregion

        #region Products

        /// <summary>
        /// Get Products by the Category Name
        /// </summary>
        /// <param name="CategoryName"></param>
        /// <returns>Array of Products</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Products GetProductByName(String ProductName);

        /// <summary>
        /// Get Products by the Category Name
        /// </summary>
        /// <param name="CategoryName"></param>
        /// <returns>Array of Products</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Products GetProductByProductID(int ProductID);
        /// <summary>
        /// Get Products by the Category Name
        /// </summary>
        /// <param name="CategoryName"></param>
        /// <returns>Array of Products</returns>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Products [] GetProductsByCategoryName(String CategoryName);

        /// <summary>
        /// Get Products by the Category ID
        /// </summary>
        /// <param name="CategoryID">Array of Products</param>
        [OperationContract]
        [FaultContract(typeof(ServiceFault))]
        [FaultContract(typeof(ServiceValidationFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Products[] GetProductsByCategoryID( int CategoryID );

        /// <summary>
        /// Get all Product Categories
        /// </summary>
        [OperationContract]
        [FaultContract(typeof(ServiceValidationFault))]
        [FaultContract(typeof(ServiceFault))]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        Categories[] GetCategories();

        #endregion

    }
}


