﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Firma.Anwendung.BusinessEntities;
using System.Data.Common;
using System.Data;
using System.Collections;
using Firma.Anwendung.Utility;



#pragma warning disable 168

namespace Firma.Anwendung.Services
{

    public class ProductServiceDB : ETBaseServiceDB
    {

        //stored Procedures:
        private const string SPGetProductByName = "spProduct_GetProductByName";
        private const string SPGetProductByProductID = "spProduct_GetProductByProductID";
        private const string SPGetProductsByName = "spProduct_GetProductsByName";
        private const string SPGetProductsByID = "spProduct_GetProductsByID";
        private const string SPGetCategories = "spProduct_GetCategories";

        /// <summary>
        /// The name of the service.
        /// </summary>
        protected override string ServiceName
        {
            get
            {
                return "ProductService";
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public ProductServiceDB()
            : base()
        {
            m_oLogUtility = new LogUtility(typeof(ProductServiceDB));
        }


        /// <summary>
        /// GetProductByProductIDDB
        /// </summary>
        /// <param name="productID"></param>
        /// <returns>Products</returns>
        internal Products GetProductByProductIDDB(int productID)
        {
            Products products = null;
            DbConnection connection = null;
            DbCommand command = null;
            IDataReader reader = null;

            try
            {
                command = Database.GetStoredProcCommand(SPGetProductByProductID);
                Database.AddInParameter(command, "@productID", DbType.Int32, productID);
                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = Database.ExecuteReader(command);
                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    reader.Read();
                        MapReaderToProduct(reader, products = new Products());
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return products;
        }

        /// <summary>
        /// GetProductByNameDB
        /// </summary>
        /// <param name="productID"></param>
        /// <returns>Products</returns>
        internal Products GetProductByNameDB(string productName)
        {
            Products products = null;
            DbConnection connection = null;
            DbCommand command = null;
            IDataReader reader = null;

            try
            {
                command = Database.GetStoredProcCommand(SPGetProductByName);
                Database.AddInParameter(command, "@productName", DbType.String, productName);
                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    reader.Read();
                    MapReaderToProduct(reader, products = new Products());
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return products;
        }

        /// <summary>
        /// GetProductByCategoryNameDB
        /// </summary>
        /// <param name="CategoryName"></param>
        /// <returns>Products[]</returns>
        internal Products [] GetProductsByCategoryNameDB(string CategoryName)
        {
            ArrayList cProductsList = new ArrayList();
            DbConnection connection = null;
            DbCommand command = null;
            IDataReader reader = null;
            Products custprod = null;


            try
            {
                command = Database.GetStoredProcCommand(SPGetProductsByName);
                Database.AddInParameter(command, "@categoryName", DbType.String, CategoryName);
                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    cProductsList.Capacity = 100;
                    for (int i = 0; reader.Read(); i++)
                    {
                        MapReaderToProduct(reader, custprod = new Products());
                        cProductsList.Add(custprod);
                    }
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return (Products[])cProductsList.ToArray(typeof(Products));
        }

        /// <summary>
        /// GetProductByCategoryIDDB
        /// </summary>
        /// <param name="CategoryID"></param>
        /// <returns>Products[]</returns>
        internal Products[] GetProductsByCategoryIDDB(int CategoryID)
        {
            ArrayList cProductsList = new ArrayList();
            DbConnection connection = null;
            DbCommand command = null;
            IDataReader reader = null;
            Products custprod = null;


            try
            {
                command = Database.GetStoredProcCommand(SPGetProductsByID);
                Database.AddInParameter(command, "@categoryID", DbType.Int32, CategoryID);
                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    cProductsList.Capacity = 100;
                    for (int i = 0; reader.Read(); i++)
                    {
                        MapReaderToProduct(reader, custprod = new Products());
                        cProductsList.Add(custprod);
                    }
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return (Products[]) cProductsList.ToArray(typeof(Products));
        }

        /// <summary>
        /// GeCategories
        /// </summary>
        /// <returns>Categories[]</returns>

        internal Categories[] GetCategoriesDB()
        {
            ArrayList cCategoriesList = new ArrayList();
            DbConnection connection = null;
            DbCommand command = null;
            IDataReader reader = null;
            Categories curcat = null;


            try
            {
                command = Database.GetStoredProcCommand(SPGetCategories);
                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    cCategoriesList.Capacity = 100;
                    for (int i = 0; reader.Read(); i++)
                    {
                        MapReaderToCategory(reader, curcat = new Categories());
                        cCategoriesList.Add(curcat);
                    }
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return (Categories[])cCategoriesList.ToArray(typeof(Categories));

        }

        private static void MapReaderToProduct(IDataReader reader, Products cProduct)
        {
            // Map Database Fields to Customer Object
            cProduct.StaticDataId = reader.GetInt32(0);           // ProductID
            if (!reader.IsDBNull(1))
                cProduct.ProductName = reader.GetString(1);
            if (!reader.IsDBNull(2))
                cProduct.CategoryID = reader.GetInt32(2);
            if (!reader.IsDBNull(3))
                cProduct.QuantityPerUnit = reader.GetString(3);
            if (!reader.IsDBNull(4))
                cProduct.UnitPrice = reader.GetDecimal(4);
            if (!reader.IsDBNull(5))
                cProduct.UnitsInStock = reader.GetInt16(5);
            if (!reader.IsDBNull(6))
                cProduct.UnitsOnOrder = reader.GetInt16(6);
            if (!reader.IsDBNull(7))
                cProduct.ReorderLevel = reader.GetInt16(7);
            if (!reader.IsDBNull(8))
                cProduct.Discontinued = reader.GetBoolean(8);
        }

        private static void MapReaderToCategory(IDataReader reader, Categories category)
        {
            // Map Database Fields to Customer Object
            category.StaticDataId = reader.GetInt32(0);           // CategoryID
            if (!reader.IsDBNull(1))
                category.CategoryName = reader.GetString(1);
            if (!reader.IsDBNull(2))
                category.Description = reader.GetString(2);
        }


    }
}
