﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using Firma.Anwendung.BusinessEntities;
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Data;
using System.Data.Common;
using Firma.Anwendung.Utility;


namespace Firma.Anwendung.Services
{
    /// <summary>
    /// class ProductService
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall, IncludeExceptionDetailInFaults = true, Name="ProductService",
        Namespace = "http://samples.microsoft.com/ServiceModel/Relay/")]
    public class ProductService : ETBaseService, IProductService
    {
        protected override string ServiceName
        {
            get
            {
                return "ProductService";
            }
        }

        private ProductServiceDB m_oProductServiceDB = null;
        /// <summary>
        /// The data access object
        /// </summary>
        public ProductServiceDB ProductServiceDb
        {
            get
            {
                if (m_oProductServiceDB == null)
                {
                    m_oProductServiceDB = new ProductServiceDB();
                }
                return m_oProductServiceDB;
            }
            set
            {
                m_oProductServiceDB = value;
            }
        }


        #region IProductService Members

        /// <summary>
        /// GetServiceInfo
        /// </summary>
        /// <returns> 
        /// class ServiceInfo
        /// </returns>
        public override Firma.Anwendung.BusinessEntities.ServiceInfo GetServiceInfo()
        {
            ServiceInfo oServiceInfo = new ServiceInfo();
            oServiceInfo.ServiceName = GetType().FullName;
            oServiceInfo.ServiceVersion = "$VERSION$";

            return oServiceInfo;
        }

        /// <summary>
        /// GetProductByProductID
        /// </summary>
        /// <param name="ProductID"></param>
        /// <returns>Products</returns>
        public Products GetProductByProductID(int ProductID)
        {
            Products oProducts;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (ProductID == 0)
                {
                    validationResult.Success = false;
                    validationResult.Message = "ProductID == 0";
                    throw new ValidationException(validationResult);
                }

                oProducts = ProductServiceDb.GetProductByProductIDDB(ProductID);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return oProducts;
        }


        /// <summary>
        /// GetProductByName
        /// </summary>
        /// <param name="ProductName"></param>
        /// <returns>Products</returns>
        public Products GetProductByName(String ProductName)
        {
            Products oProducts;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (ProductName == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "ProductName == null";
                    throw new ValidationException(validationResult);
                }
                if (ProductName.Length == 0 )
                {
                    validationResult.Success = false;
                    validationResult.Message = "ProductName Length = 0";
                    throw new ValidationException(validationResult);
                }

                oProducts = ProductServiceDb.GetProductByNameDB(ProductName);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return oProducts;
        }

        /// <summary>
        /// GetProductByCategoryName
        /// </summary>
        /// <param name="CategoryName"></param>
        /// <returns>All available products for a CategoryName</returns>
        Products[] IProductService.GetProductsByCategoryName(string CategoryName)
        {
            Products[] oProducts;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (CategoryName == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "CategoryName == null";
                    throw new ValidationException(validationResult);
                }
                if (CategoryName.Length == 0 )
                {
                    validationResult.Success = false;
                    validationResult.Message = "CategoryName Length = 0";
                    throw new ValidationException(validationResult);
                }

                oProducts = ProductServiceDb.GetProductsByCategoryNameDB(CategoryName);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return oProducts;
        }

        /// <summary>
        /// GetProductByCategoryID
        /// </summary>
        /// <param name="CategoryID"></param>
        /// <returns>All available products for a CategoryID</returns>
        Products[] IProductService.GetProductsByCategoryID(int CategoryID)
        {
            Products[] oProducts;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (CategoryID == 0)
                {
                    validationResult.Success = false;
                    validationResult.Message = "CategoryID = 0";
                    throw new ValidationException(validationResult);
                }

                oProducts = ProductServiceDb.GetProductsByCategoryIDDB(CategoryID);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return oProducts;
        }

        /// <summary>
        /// GetCategories
        /// </summary>
        /// <returns>All available Categories as Array</returns>
        Categories[] IProductService.GetCategories()
        {
            Categories[] oCategories;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                oCategories = ProductServiceDb.GetCategoriesDB();
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return oCategories;
        }

        #endregion
    }
}

