﻿using System;
using System.Collections.Generic;
using System.IO;
using System.ServiceModel;
using Firma.Anwendung.BusinessEntities;
using Firma.Anwendung.Utility;
using System.Text;


namespace Firma.Anwendung.Services
{
    /// <summary>
    ///  Base Class for all Services
    /// </summary>
    abstract public class ETBaseService : IETBaseService
    {
        /// <summary>
        /// LogUtility
        /// </summary>
        protected LogUtility m_oLogUtility = null;

        /// <summary>
        /// IServiceProxy
        /// </summary>
        protected IServiceProxy m_oIServiceProxy = null;
                /// <summary>
        /// The name of the service.
        /// </summary>
        protected abstract string ServiceName
        {
            get;
        }

        /// <summary>
        /// ETBaseService
        /// </summary>
        /// <param name="oIServiceProxy"></param>
        public ETBaseService(IServiceProxy oIServiceProxy)
        {
            m_oIServiceProxy = oIServiceProxy;
        }

        /// <summary>
        /// The constructor.
        /// </summary>
        protected ETBaseService()
        {

        }

        /// <summary>
        /// Get information (e.g. name, version) about the service
        /// </summary>
        /// <returns>Information about the service</returns>
        public virtual ServiceInfo GetServiceInfo()
        {
            ServiceInfo oServiceInfo = new ServiceInfo();
            oServiceInfo.ServiceName =  "unknown";
            oServiceInfo.ServiceVersion = "unknown";
            return oServiceInfo;
        }
        /// <summary>
        /// Create a ValidationException for the service.
        /// </summary>
        /// <param name="oValidationException">The orign ValidationException</param>
        /// <returns>The orign ValidationException wrapped as a FaultException object.</returns>
        protected FaultException<ServiceValidationFault> CreateValidationFault(ValidationException oValidationException)
        {
            m_oLogUtility.TraceException(LogLevel.Error, "Validation Error in Service " + ServiceName + ":" + oValidationException.Message, oValidationException);
            ServiceValidationFault oServiceValidationFault = new ServiceValidationFault();
            oServiceValidationFault.Id = Guid.NewGuid();
            oServiceValidationFault.Message = oValidationException.Message;
            FaultException<ServiceValidationFault> faultException = new FaultException<ServiceValidationFault>(oServiceValidationFault, new FaultReason(oValidationException.Message));
            return faultException;
        }

        /// <summary>
        /// Create a service specific Fault exception.
        /// </summary>
        /// <param name="oException">The orign exception.</param>
        /// <returns>The orign exception wrapped as a FaultException object.</returns>
        protected FaultException<ServiceFault> CreateServiceFault(Exception oException)
        {
            m_oLogUtility.TraceException(LogLevel.Error, "Exception in Service " + ServiceName + ":" + oException.Message, oException);
            ServiceFault containerServiceFaultException = new ServiceFault();
            containerServiceFaultException.Id = Guid.NewGuid();
            containerServiceFaultException.Message = oException.Message;
            FaultException<ServiceFault> faultException =
                new FaultException<ServiceFault>(containerServiceFaultException, new FaultReason(oException.Message));
            return faultException;
        }

        /// <summary>
        /// Check for presence of Service Config File
        /// </summary>
        /// <returns>true if available</returns>
        public bool CheckServiceConfiguration()
        {
            bool result = false;
            string servicepath = ETUtility.GetServiceRootPath();
            StringBuilder sb = new StringBuilder(servicepath, 100);
            sb.Append("bin\\");
            sb.Append(ServiceName);
            sb.Append(".dll.config");

            if (File.Exists(sb.ToString()))
            {
                result = true;
            }
            return result;
        }

    }
}
