﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.ServiceModel;
using System.IO;
using Firma.Anwendung.BusinessEntities;
using Firma.Anwendung.Utility;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;

namespace Firma.Anwendung.Services
{
    /// <summary>
    /// Base Class for Database Services
    /// </summary>
    public abstract class ETBaseServiceDB
    {
        /// <summary>
        /// The database instance of the service
        /// </summary>
        private Database m_oDatabase = null;

        /// <summary>
        /// The Logger.
        /// </summary>
        protected LogUtility m_oLogUtility = null;

        /// <summary>
        /// The name of the service.
        /// </summary>
        protected abstract string ServiceName
        {
            get;
        }
        /// <summary>
        /// The database.
        /// </summary>
        public Database Database
        {
            get
            {
                if (m_oDatabase == null)
                {
                    Assembly oAssembly = Assembly.GetAssembly(typeof(ETBaseServiceDB));

                    Uri oUri = new Uri(oAssembly.CodeBase);
                    if (!File.Exists(oUri.LocalPath))
                    {
                        throw new ArgumentNullException(string.Format("Configfile '{0}' missing", oUri.LocalPath));
                    }

                    string strConfigFile = Path.GetDirectoryName(oUri.LocalPath) + "\\" + ServiceName + ".dll.config";
                    if (File.Exists(strConfigFile))
                    {
                        m_oDatabase = ETUtility.GetDefaultDatabase(strConfigFile);
                    }
                    else
                    {
                        m_oLogUtility.TraceLog(LogLevel.Error, "Missing Configfile: " + strConfigFile + "!");
                    }
                }

                return m_oDatabase;
            }
        }


    }
}
