﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using Firma.Anwendung.BusinessEntities;
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Data;
using System.Data.Common;
using Firma.Anwendung.Utility;



namespace Firma.Anwendung.Services
{
    /// <summary>
    /// class CustomerService
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall, IncludeExceptionDetailInFaults = true)]
    public class CustomerService : ETBaseService, ICustomerService
    {
        //consts
        private const int CustomerIDLength = 5;

        /// <summary>
        /// The name of the service.
        /// </summary>
        protected override string ServiceName
        {
            get
            {
                return "CustomerService";
            }
        }

        private CustomerServiceDB m_cCustomerServiceDB = null;
        /// <summary>
        /// The data access object
        /// </summary>
        public CustomerServiceDB CustomerSeriveDb
        {
            get
            {
                if (m_cCustomerServiceDB == null)
                {
                    m_cCustomerServiceDB = new CustomerServiceDB();
                }
                return m_cCustomerServiceDB;
            }
            set
            {
                m_cCustomerServiceDB = value;
            }
        }

        #region ICustomerService Members

        /// <summary>
        /// GetServiceInfo
        /// </summary>
        /// <returns> 
        /// class ServiceInfo
        /// </returns>
        public override Firma.Anwendung.BusinessEntities.ServiceInfo GetServiceInfo()
        {
            ServiceInfo oServiceInfo = new ServiceInfo();
            oServiceInfo.ServiceName = GetType().FullName;
            oServiceInfo.ServiceVersion = "$VERSION$";

            return oServiceInfo;
        }
        /// <summary>
        /// CreateCustomer
        /// </summary>
        /// <param name="customer"></param>
        [OperationBehavior(TransactionScopeRequired = true)]
        public void CreateCustomer(Firma.Anwendung.BusinessEntities.Customer customer)
        {

            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (customer.StaticDataId == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "customer.StaticDataId==null";
                    throw new ValidationException(validationResult);
                }
                if (customer.StaticDataId.Length != CustomerIDLength)
                {
                    validationResult.Success = false;
                    validationResult.Message = "customer.StaticDataId Length != 5";
                    throw new ValidationException(validationResult);
                }
                if (customer.CompanyName == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "customer.CompanyName==null";
                    throw new ValidationException(validationResult);
                }
                CustomerSeriveDb.CreateCustomerDB(customer);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
        }

        /// <summary>
        /// UpdateCustomer
        /// </summary>
        /// <param name="new_customer"></param>
        /// <param name="old_customer"></param>
        [OperationBehavior(TransactionScopeRequired = true)]
        public void UpdateCustomer(Customer new_customer, Customer old_customer)
        {
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (old_customer.StaticDataId == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "old_customer.StaticDataId==null";
                    throw new ValidationException(validationResult);
                }
                if (old_customer.StaticDataId.Length != CustomerIDLength)
                {
                    validationResult.Success = false;
                    validationResult.Message = "old_customer.StaticDataId Length != 5";
                    throw new ValidationException(validationResult);
                }
                if (old_customer.StaticDataId != new_customer.StaticDataId)
                {
                    validationResult.Success = false;
                    validationResult.Message = "old_customer.StaticDataId != new_customer.StaticDataId";
                    throw new ValidationException(validationResult);
                }
                if (old_customer.CompanyName == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "old_customer.CompanyName==null";
                    throw new ValidationException(validationResult);
                }
                if (new_customer.CompanyName == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "new_customer.CompanyName==null";
                    throw new ValidationException(validationResult);
                }
                CustomerSeriveDb.UpdateCustomerDB(new_customer, old_customer);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
        }

        /// <summary>
        /// DeleteCustomer
        /// </summary>
        /// <param name="customerId"></param>
        [OperationBehavior(TransactionScopeRequired = true)]
        public void DeleteCustomer(string customerId)
        {
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (customerId == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "customerId==null";
                    throw new ValidationException(validationResult);
                }
                if (customerId.Length != CustomerIDLength)
                {
                    validationResult.Success = false;
                    validationResult.Message = "customerId Length != 5";
                    throw new ValidationException(validationResult);
                }
                CustomerSeriveDb.DeleteCustomerDB(customerId);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
        }
        /// <summary>
        /// GetCustumerById
        /// </summary>
        /// <param name="customerId"></param>
        /// <returns>
        /// Customer class
        /// </returns>
        // [OperationBehavior(TransactionScopeRequired = true)]
        public Firma.Anwendung.BusinessEntities.Customer GetCustomerById(string customerId)
        {
            Customer cCustomer;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (customerId == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "customerId==null";
                    throw new ValidationException(validationResult);
                }
                if (customerId.Length != CustomerIDLength)
                {
                    validationResult.Success = false;
                    validationResult.Message = "customerId Length != 5";
                    throw new ValidationException(validationResult);
                }
                cCustomer = CustomerSeriveDb.GetCustomerByIdDB(customerId);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return cCustomer;
        }

        /// <summary>
        /// GetCustomerArray - all Customers in an Array
        /// </summary>
        /// <returns>
        /// Array of Customer class
        /// </returns>
        public Customer [] GetCustomerArray()
        {
            Customer[] cCustomerArray;
            try
            {
                cCustomerArray = CustomerSeriveDb.GetCustomerArrayDB();
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
            return cCustomerArray;
        }

        /// <summary>
        /// GetCustomerArray - for Companies with a CompanyName like
        /// </summary>
        /// <returns>
        /// Array of Customer class
        /// </returns>
        public Customer[] GetCustomerArrayForName(String companyName)
        {
            Customer[] cCustomerArray;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (companyName == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "companyName==null";
                    throw new ValidationException(validationResult);
                }
                if (companyName.Length == 0)
                {
                    validationResult.Success = false;
                    validationResult.Message = "companyName Empty";
                    throw new ValidationException(validationResult);
                }
                cCustomerArray = CustomerSeriveDb.GetCustomerArrayForNameDB(companyName.Replace('*', '%'));
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
            return cCustomerArray;

        }
        #endregion
 
    }
}
