﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.Common;
using System.Data.SqlTypes;
using Firma.Anwendung.BusinessEntities;
using Firma.Anwendung.Utility;

#pragma warning disable 168


namespace Firma.Anwendung.Services
{
    public class CustomerServiceDB : ETBaseServiceDB
    {
        //stored Procedures:
        private const string SPGetCustomerByID = "spCustomer_GetCustomerByID";
        private const string SPGetCustomerArray = "spCustomer_GetAllCustomers";
        private const string SPCreateCustomer = "spCustomer_InsertCustomers";
        private const string SPUpdateCustomer = "spCustomer_UpdateCustomers";
        private const string SPDeleteCustomer = "spCustomer_DeleteCustomers";
        private const string SPGetCustomerArrayForName = "spCustomer_GetCustomersForName";

        /// <summary>
        /// The name of the service.
        /// </summary>
        protected override string ServiceName
        {
            get
            {
                return "CustomerService";
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public CustomerServiceDB() : base()
        {
            m_oLogUtility = new LogUtility(typeof(CustomerServiceDB));
        }

        /// <summary>
        /// Retrieves a single Customer from the database
        /// </summary>
        /// <param name="id">The id of the order</param>
        /// <returns>A Customer with the specified Id</returns>
        internal Customer GetCustomerByIdDB(string customerID)
        {   
            Customer cCustomer = new Customer();
            DbCommand command = null;
            DbConnection connection= null;
            IDataReader reader = null;
            
            
            try
            {
                command = Database.GetStoredProcCommand(SPGetCustomerByID);
                Database.AddInParameter(command, "@customerID", DbType.String, customerID);
                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    reader.Read();
                    
                    MapReaderToCustomer(reader, cCustomer);
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if( reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return cCustomer;

        }

        /// <summary>
        /// GetCustomerArrayDB
        /// </summary>
        /// <returns>Customer []</returns>
        internal Customer [] GetCustomerArrayDB()
        {
            ArrayList cCustomerArray = new ArrayList();
            DbCommand command = null;
            DbConnection connection = null;
            IDataReader reader = null;
            Customer custtemp = null;


            try
            {
                command = Database.GetStoredProcCommand(SPGetCustomerArray);
                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    cCustomerArray.Capacity = 100;
                    for( int i = 0;  reader.Read(); i++ )
                    {
                        MapReaderToCustomer(reader, custtemp = new Customer());
                        cCustomerArray.Add(custtemp);
                    }
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return (Customer[]) cCustomerArray.ToArray(typeof(Customer)) ;
           
        }

        /// <summary>
        /// GetCustomerArrayDB
        /// </summary>
        /// <returns>Customer []</returns>
        internal Customer[] GetCustomerArrayForNameDB(string companyName)
        {
            ArrayList cCustomerArray = new ArrayList();
            DbCommand command = null;
            DbConnection connection = null;
            IDataReader reader = null;
            Customer custtemp = null;


            try
            {
                command = Database.GetStoredProcCommand(SPGetCustomerArrayForName);
                Database.AddInParameter(command, "@companyName", DbType.String, companyName);

                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    cCustomerArray.Capacity = 100;
                    for (int i = 0; reader.Read(); i++)
                    {
                        MapReaderToCustomer(reader, custtemp = new Customer());
                        cCustomerArray.Add(custtemp);
                    }
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return (Customer[])cCustomerArray.ToArray(typeof(Customer));

        }


        /// <summary>
        /// CreateCustomerDB
        /// </summary>
        /// <param name="customer">Customer</param>
        internal void CreateCustomerDB(Customer customer)
        {
            DbCommand command = null;
            DbConnection connection = null;


            try
            {
                command = Database.GetStoredProcCommand(SPCreateCustomer);
                Database.AddInParameter(command, "@customerID", DbType.String, customer.StaticDataId);
                Database.AddInParameter(command, "@companyName", DbType.String, customer.CompanyName);
                Database.AddInParameter(command, "@contactName", DbType.String, customer.ContactName);
                Database.AddInParameter(command, "@contactTitle", DbType.String, customer.ContactTitle);
                Database.AddInParameter(command, "@address", DbType.String, customer.Address);
                Database.AddInParameter(command, "@city", DbType.String, customer.City);
                Database.AddInParameter(command, "@postalCode", DbType.String, customer.PostalCode);
                Database.AddInParameter(command, "@country", DbType.String, customer.Country);
                Database.AddInParameter(command, "@phone", DbType.String, customer.Phone);
                Database.AddInParameter(command, "@fax", DbType.String, customer.Fax);

                connection = Database.CreateConnection();
                command.Connection = connection;


                ETUtility.ExecuteNonQuery(Database, command);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
        }

        /// <summary>
        /// UpdateCustomerDB
        /// </summary>
        /// <param name="new_customer"></param>
        /// <param name="old_customer"></param>
        internal void UpdateCustomerDB(Customer new_customer, Customer old_customer)
        {
            DbCommand command = null;
            DbConnection connection = null;


            try
            {
                command = Database.GetStoredProcCommand(SPUpdateCustomer);
                Database.AddInParameter(command, "@customerID", DbType.String, new_customer.StaticDataId);

                Database.AddInParameter(command, "@companyName", DbType.String, new_customer.CompanyName);
                Database.AddInParameter(command, "@before_companyName", DbType.String, old_customer.CompanyName);

                Database.AddInParameter(command, "@contactName", DbType.String, new_customer.ContactName);
                Database.AddInParameter(command, "@before_contactName", DbType.String, old_customer.ContactName);
                Database.AddInParameter(command, "@contactTitle", DbType.String, new_customer.ContactTitle);
                Database.AddInParameter(command, "@before_contactTitle", DbType.String, old_customer.ContactTitle);
                Database.AddInParameter(command, "@address", DbType.String, new_customer.Address);
                Database.AddInParameter(command, "@before_address", DbType.String, old_customer.Address);
                Database.AddInParameter(command, "@city", DbType.String, new_customer.City);
                Database.AddInParameter(command, "@before_city", DbType.String, old_customer.City);
                Database.AddInParameter(command, "@postalCode", DbType.String, new_customer.PostalCode);
                Database.AddInParameter(command, "@before_postalCode", DbType.String, old_customer.PostalCode);
                Database.AddInParameter(command, "@country", DbType.String, new_customer.Country);
                Database.AddInParameter(command, "@before_country", DbType.String, old_customer.Country);
                Database.AddInParameter(command, "@phone", DbType.String, new_customer.Phone);
                Database.AddInParameter(command, "@before_phone", DbType.String, old_customer.Phone);
                Database.AddInParameter(command, "@fax", DbType.String, new_customer.Fax);
                Database.AddInParameter(command, "@before_fax", DbType.String, old_customer.Fax);

                connection = Database.CreateConnection();
                command.Connection = connection;


                ETUtility.ExecuteNonQuery(Database, command);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
        }

        /// <summary>
        /// DeleteCustomerDB
        /// </summary>
        /// <param name="dCustomerID"></param>
        internal void DeleteCustomerDB(string dCustomerID)
        {
            DbCommand command = null;
            DbConnection connection = null;


            try
            {
                command = Database.GetStoredProcCommand(SPDeleteCustomer);
                Database.AddInParameter(command, "@customerID", DbType.String, dCustomerID);
                connection = Database.CreateConnection();
                command.Connection = connection;


                ETUtility.ExecuteNonQuery(Database, command);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }

        }
        
        /// <summary>
        /// MapReaderToCustomer
        /// </summary>
        /// <param name="reader"></param>
        /// <param name="cCustomer"></param>
        private static void MapReaderToCustomer(IDataReader reader, Customer cCustomer)
        {
            // Map Database Fields to Customer Object
            cCustomer.StaticDataId = reader["CustomerID"] as string;
            cCustomer.CompanyName = reader["CompanyName"] as string;
            cCustomer.ContactName = reader["ContactName"] as string;
            cCustomer.ContactTitle = reader["ContactTitle"] as string;
            cCustomer.Address = reader["Address"] as string;
            cCustomer.City = reader["City"] as string;
            cCustomer.PostalCode = reader["PostalCode"] as string;
            cCustomer.Country = reader["Country"] as string;
            cCustomer.Phone = reader["Phone"] as string;
            cCustomer.Fax = reader["Fax"] as string;
        }


   }
}
