﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using Firma.Anwendung.BusinessEntities;
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Data;
using System.Data.Common;
using Firma.Anwendung.Utility;



namespace Firma.Anwendung.Services
{
    /// <summary>
    /// class OrderService
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall, IncludeExceptionDetailInFaults = true, ConcurrencyMode = ConcurrencyMode.Single,
                TransactionIsolationLevel = System.Transactions.IsolationLevel.Serializable)]
    public class OrderService : ETBaseService, IOrderService
    {
        private const int CustomerIDLength = 5;

        protected override string ServiceName
        {
            get
            {
                return "OrderService";
            }
        }

        private OrderServiceDB m_oOrderServiceDB = null;
        /// <summary>
        /// The data access object
        /// </summary>
        public OrderServiceDB OrderServiceDb
        {
            get
            {
                if (m_oOrderServiceDB == null)
                {
                    m_oOrderServiceDB = new OrderServiceDB();
                }
                return m_oOrderServiceDB;
            }
            set
            {
                m_oOrderServiceDB = value;
            }
        }

        #region IOrderService Members

        /// <summary>
        /// GetServiceInfo
        /// </summary>
        /// <returns> 
        /// class ServiceInfo
        /// </returns>
        public override Firma.Anwendung.BusinessEntities.ServiceInfo GetServiceInfo()
        {
            ServiceInfo oServiceInfo = new ServiceInfo();
            oServiceInfo.ServiceName = GetType().FullName;
            oServiceInfo.ServiceVersion = "$VERSION$";

            return oServiceInfo;
        }

        /// <summary>
        /// CreateOrder
        /// </summary>
        /// <param name="order"></param>
        /// <returns>OrderID of the new row as int</returns>
        public int CreateOrder(Order order)
        {
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (order.CustomerID == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "order.CustomerID == null";
                    throw new ValidationException(validationResult);
                }
                if (order.CustomerID.Length != CustomerIDLength)
                {
                    validationResult.Success = false;
                    validationResult.Message = "order.CustomerID Length != 5";
                    throw new ValidationException(validationResult);
                }
                order.StaticDataId = OrderServiceDb.CreateOrderDB(order);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
            return order.StaticDataId;
        }

        /// <summary>
        /// UpdateOrder
        /// </summary>
        /// <param name="new_order"></param>
        /// <param name="old_order"></param>
        public void UpdateOrder(Order new_order, Order old_order)
        {
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (old_order.StaticDataId == 0)
                {
                    validationResult.Success = false;
                    validationResult.Message = "old_order.StaticDataId==null";
                    throw new ValidationException(validationResult);
                }
                if (old_order.StaticDataId != new_order.StaticDataId)
                {
                    validationResult.Success = false;
                    validationResult.Message = "old_customer.StaticDataId != new_customer.StaticDataId";
                    throw new ValidationException(validationResult);
                
                }
                OrderServiceDb.UpdateOrderDB(new_order, old_order);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
        }


        /// <summary>
        /// DeleteOrder
        /// </summary>
        /// <param name="orderId"></param>
        public void DeleteOrder(int orderId)
        {
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (orderId == 0)
                {
                    validationResult.Success = false;
                    validationResult.Message = "Parameter orderID == 0";
                    throw new ValidationException(validationResult);
                }
                OrderServiceDb.DeleteOrderDB(orderId);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
            return ;

        }

        /// <summary>
        /// GetOrderById
        /// </summary>
        /// <param name="orderId"></param>
        /// <returns>Order with orderID</returns>
        public Order GetOrderById(int orderId)
        {
            Order oOrder;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (orderId == 0)
                {
                    validationResult.Success = false;
                    validationResult.Message = "orderId == 0";
                    throw new ValidationException(validationResult);
                }
                oOrder = OrderServiceDb.GetOrderByIdDB(orderId);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return oOrder;
        }

        /// <summary>
        /// GetOrdersforCustomer
        /// </summary>
        /// <param name="customerID"></param>
        /// <returns>All Orders customerID</returns>
        public Order[] GetOrdersforCustomer(string customerID)
        {
            Order [] oOrder;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (customerID == null)
                {
                    validationResult.Success = false;
                    validationResult.Message = "customerID == null";
                    throw new ValidationException(validationResult);
                }
                if (customerID.Length != CustomerIDLength)
                {
                    validationResult.Success = false;
                    validationResult.Message = "customer.StaticDataId Length != 5";
                    throw new ValidationException(validationResult);
                }

                oOrder = OrderServiceDb.GetOrdersforCustomerDB(customerID);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return oOrder;
        }



        /// <summary>
        /// CreateOrderDetail
        /// </summary>
        /// <param name="orderDetail"></param>
        /// <returns>Creates a new OrderDetail for OrderId</returns>
        public void CreateOrderDetail(OrderDetail orderDetail)
        {
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (orderDetail.IsValid() == false)
                {
                    validationResult.Success = false;
                    validationResult.Message = "orderDetail not Valid!";
                    throw new ValidationException(validationResult);
                }
                OrderServiceDb.CreateOrderDetailDB(orderDetail);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
        }


        /// <summary>
        /// UpdateOrderDetail
        /// </summary>
        /// <param name="new_orderDetail"></param>
        /// <param name="old_orderDetail"></param>
        /// <returns>Updates an OrderDetail for OrderId</returns> 
        public void UpdateOrderDetail(OrderDetail new_orderDetail, OrderDetail old_orderDetail)
        {
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (new_orderDetail.IsValid() == false)
                {
                    validationResult.Success = false;
                    validationResult.Message = "new_orderDetail not Valid!";
                    throw new ValidationException(validationResult);
                }
                OrderServiceDb.UpdateOrderDetailDB(new_orderDetail, old_orderDetail);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
        

        }


        /// <summary>
        /// GetOrderDetailsByOrderID
        /// </summary>
        /// <param name="orderID"></param>
        /// <returns>All OrderDetails for orderID</returns>
        public OrderDetail[] GetOrderDetailsByOrderID(int orderID)
        {
            OrderDetail[] oOrderDetail;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (orderID == 0)
                {
                    validationResult.Success = false;
                    validationResult.Message = "orderID == 0";
                    throw new ValidationException(validationResult);
                }

                oOrderDetail = OrderServiceDb.GetOrderDetailsByOrderIDDB(orderID);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return oOrderDetail;

        }

        /// <summary>
        /// GetOrderandProductDetailsByOrderID
        /// </summary>
        /// <param name="orderID"></param>
        /// <returns>All OrderDetailProductDetails for orderID</returns>
        public OrderDetailProductDetail[] GetOrderandProductDetailsByOrderID(int orderID)
        {
            OrderDetailProductDetail[] oOrderDetail;
            ValidationResult validationResult = new ValidationResult();
            try
            {
                if (orderID == 0)
                {
                    validationResult.Success = false;
                    validationResult.Message = "orderID == 0";
                    throw new ValidationException(validationResult);
                }

                oOrderDetail = OrderServiceDb.GetOrderandProductDetailsByOrderIDDB(orderID);
            }
            catch (ValidationException ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateValidationFault(ex);
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw CreateServiceFault(ex);
            }
            finally
            {
            }

            return oOrderDetail;

        }

        /// <summary>
        /// CreateOrderWithOrderDetails
        /// </summary>
        /// <param name="order"></param>
        /// <param name="orderDetails[]"></param>
        /// <returns>OrderID od the newly created Order</returns>
        [OperationBehavior(TransactionAutoComplete=true, TransactionScopeRequired=true)]
        public int CreateOrderWithOrderDetails(Order order, OrderDetail[] orderDetails)
        {
            int newOrderID = 0;
            int count = 0;
            try
            {
                // create Order and retrieve new OrderID
                newOrderID = CreateOrder(order);
                foreach (OrderDetail od in orderDetails)
                {
                    // set new OrderID for each Order Detail
                    od.OrderID = newOrderID;
                    CreateOrderDetail(od);
                    count++;
                }
            }
            catch (Exception ex)
            {
                throw CreateServiceFault(ex);
            }
            finally
            {
            }
            return newOrderID;
        }

        #endregion

    }
}
