﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.Common;
using System.Data.SqlTypes;
using Firma.Anwendung.BusinessEntities;
using Firma.Anwendung.Utility;

#pragma warning disable 168


namespace Firma.Anwendung.Services
{
    public class OrderServiceDB : ETBaseServiceDB
    {
        //stored Procedures:
        private const string SPGetOrderByID = "spOrder_GetOrderByID";
        private const string SPGetOrderforCustomer = "spOrder_GetOrdersforCustomer";
        private const string SPCreateOrder = "spOrder_InsertOrder";
        private const string SPUpdateOrder = "spOrder_UpdateOrder";
        private const string SPDeleteOrder = "spOrder_DeleteOrder";
        private const string SPCreateOrderDetail = "spOrder_CreateOrderDetail";
        private const string SPGetOrderDeatilsByOrderID = "spOrder_GetOrderDetailsByOrderID";
        private const string SPGetOrderandProductDeatilsByOrderID = "spOrder_GetOrderDetailsProductDetails";
        private const string SPUpdateOrderDetail = "spOrder_UpdateOrderDetail";


        /// <summary>
        /// The name of the service.
        /// </summary>
        protected override string ServiceName
        {
            get
            {
                return "OrderService";
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public OrderServiceDB() : base()
        {
            m_oLogUtility = new LogUtility(typeof(OrderServiceDB));
        }

        /// <summary>
        /// GetOrderByIdDB
        /// </summary>
        /// <param name="orderId"></param>
        /// <returns>Order</returns>
        internal Order GetOrderByIdDB(int orderId)
        {
            Order oOrder = new Order();
            DbCommand command = null;
            DbConnection connection = null;
            IDataReader reader = null;


            try
            {
                command = Database.GetStoredProcCommand(SPGetOrderByID);
                Database.AddInParameter(command, "@orderId", DbType.Int32, orderId);
                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    reader.Read();
                    MapReaderToOrder(reader, oOrder);
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return oOrder;
        }

        /// <summary>
        /// GetOrdersforCustomerDB 
        /// </summary>
        /// <param name="customerID"></param>
        /// <returns>array of Orders</returns>
        internal Order[] GetOrdersforCustomerDB(string customerID)
        {
            ArrayList oOrderArray = new ArrayList();
            DbCommand command = null;
            DbConnection connection = null;
            IDataReader reader = null;
            Order ordertemp = null;


            try
            {
                command = Database.GetStoredProcCommand(SPGetOrderforCustomer);
                Database.AddInParameter(command, "@customerId", DbType.String, customerID);

                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    oOrderArray.Capacity = 100;
                    for (int i = 0; reader.Read(); i++)
                    {
                        MapReaderToOrder(reader, ordertemp = new Order());
                        oOrderArray.Add(ordertemp);
                    }
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return (Order[])oOrderArray.ToArray(typeof(Order));
        }

        /// <summary>
        /// CreateOrderDB
        /// </summary>
        /// <param name="order"></param>
        /// <returns>OrderID of the new row as int</returns>
        internal int CreateOrderDB(Order order)
        {
            DbCommand command = null;
            DbConnection connection = null;

            try
            {
                command = Database.GetStoredProcCommand(SPCreateOrder);
                Database.AddOutParameter(command, "@PKId", DbType.Int32, order.StaticDataId);
                Database.AddInParameter(command, "@customerID", DbType.String, order.CustomerID);
                Database.AddInParameter(command, "@employeeID", DbType.Int32, order.EmployeeId);

                if (order.OrderDate == DateTime.MinValue)
                {
                    Database.AddInParameter(command, "@orderDate", DbType.DateTime, System.DBNull.Value);
                }
                else
                {
                    Database.AddInParameter(command, "@orderDate", DbType.DateTime, order.OrderDate);
                }

                if (order.RequiredDate == DateTime.MinValue)
                {
                    Database.AddInParameter(command, "@requiredDate", DbType.DateTime, System.DBNull.Value);
                }
                else
                {
                    Database.AddInParameter(command, "@requiredDate", DbType.DateTime, order.RequiredDate);
                }

                if (order.ShippedDate == DateTime.MinValue)
                {
                    Database.AddInParameter(command, "@shippedDate", DbType.DateTime, System.DBNull.Value);
                }
                else
                {
                    Database.AddInParameter(command, "@shippedDate", DbType.DateTime, order.ShippedDate);
                }
                Database.AddInParameter(command, "@freight", DbType.Currency, order.Freight);
                Database.AddInParameter(command, "@shipName", DbType.String, order.ShipName);
                Database.AddInParameter(command, "@shipAddress", DbType.String, order.ShipAddress);
                Database.AddInParameter(command, "@shipCity", DbType.String, order.ShipCity);
                Database.AddInParameter(command, "@shipRegion", DbType.String, order.ShipRegion);
                Database.AddInParameter(command, "@shipPostalCode", DbType.String, order.ShipPostalCode);
                Database.AddInParameter(command, "@shipCountry", DbType.String, order.ShipCountry);

                connection = Database.CreateConnection();
                command.Connection = connection;


                ETUtility.ExecuteNonQuery(Database, command);
                
                order.StaticDataId = (int) command.Parameters["@PKId"].Value;
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return order.StaticDataId;
        }

        /// <summary>
        /// DeleteOrderDB
        /// </summary>
        /// <param name="orderID"></param>
        internal void DeleteOrderDB(int orderID)
        {
            DbCommand command = null;
            DbConnection connection = null;

            try
            {
                command = Database.GetStoredProcCommand(SPDeleteOrder);
                Database.AddInParameter(command, "@orderID", DbType.Int32, orderID);

                connection = Database.CreateConnection();
                command.Connection = connection;


                ETUtility.ExecuteNonQuery(Database, command);

            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return;
        }


        /// <summary>
        /// UpdateOrderDB
        /// </summary>
        /// <param name="new_order"></param>
        /// <param name="old_order"></param>
        internal void UpdateOrderDB(Order new_order, Order old_order)
        {
            DbCommand command = null;
            DbConnection connection = null;

            try
            {
                command = Database.GetStoredProcCommand(SPUpdateOrder);
                Database.AddInParameter(command, "@orderId", DbType.Int32, new_order.StaticDataId);

                Database.AddInParameter(command, "@customerID", DbType.String, new_order.CustomerID);
                Database.AddInParameter(command, "@before_customerID", DbType.String, old_order.CustomerID);
                Database.AddInParameter(command, "@employeeID", DbType.Int32, new_order.EmployeeId);
                Database.AddInParameter(command, "@before_employeeID", DbType.Int32, old_order.EmployeeId);
                Database.AddInParameter(command, "@orderDate", DbType.DateTime, new_order.OrderDate);
                Database.AddInParameter(command, "@before_orderDate", DbType.DateTime, old_order.OrderDate);
                Database.AddInParameter(command, "@requiredDate", DbType.DateTime, new_order.RequiredDate);
                Database.AddInParameter(command, "@before_requiredDate", DbType.DateTime, old_order.RequiredDate);
                Database.AddInParameter(command, "@shippedDate", DbType.DateTime, new_order.ShippedDate);
                Database.AddInParameter(command, "@before_shippedDate", DbType.DateTime, old_order.ShippedDate);
                Database.AddInParameter(command, "@shipName", DbType.String, new_order.ShipName);
                Database.AddInParameter(command, "@before_shipName", DbType.String, old_order.ShipName);
                Database.AddInParameter(command, "@shipAddress", DbType.String, new_order.ShipAddress);
                Database.AddInParameter(command, "@before_shipAddress", DbType.String, old_order.ShipAddress);
                Database.AddInParameter(command, "@shipCity", DbType.String, new_order.ShipCity);
                Database.AddInParameter(command, "@before_shipCity", DbType.String, old_order.ShipCity);
                Database.AddInParameter(command, "@shipRegion", DbType.String, new_order.ShipRegion);
                Database.AddInParameter(command, "@before_shipRegion", DbType.String, old_order.ShipRegion);
                Database.AddInParameter(command, "@shipCountry", DbType.String, new_order.ShipCountry);
                Database.AddInParameter(command, "@before_shipCountry", DbType.String, old_order.ShipCountry);
                Database.AddInParameter(command, "@shipPostalCode", DbType.String, new_order.ShipPostalCode);
                Database.AddInParameter(command, "@before_shipPostalCode", DbType.String, old_order.ShipPostalCode);
                Database.AddInParameter(command, "@freight", DbType.Currency, new_order.Freight);
                Database.AddInParameter(command, "@before_freight", DbType.Currency, old_order.Freight);
                connection = Database.CreateConnection();
                command.Connection = connection;


                ETUtility.ExecuteNonQuery(Database, command);

            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return;
        }

        /// <summary>
        /// CreateOrderDetailDB
        /// </summary>
        /// <param name="orderDetail"></param>
        internal void CreateOrderDetailDB(OrderDetail orderDetail)
        {
            DbCommand command = null;
            DbConnection connection = null;

            try
            {
                command = Database.GetStoredProcCommand(SPCreateOrderDetail);
                Database.AddInParameter(command, "@orderID", DbType.String, orderDetail.OrderID);
                Database.AddInParameter(command, "@productID", DbType.Int32, orderDetail.ProductID);
                Database.AddInParameter(command, "@unitPrice", DbType.Currency, orderDetail.UnitPrice);
                Database.AddInParameter(command, "@quantity", DbType.String, orderDetail.Quantity);
                Database.AddInParameter(command, "@discount", DbType.Single, orderDetail.Discount);

                connection = Database.CreateConnection();
                command.Connection = connection;


                ETUtility.ExecuteNonQuery(Database, command);

            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }

        }

        /// <summary>
        /// UpdateOrderDetail
        /// </summary>
        /// <param name="new_orderDetail"></param>
        /// <param name="old_orderDetail"></param>
        /// <returns>Updates an OrderDetail for OrderId</returns> 
        internal void UpdateOrderDetailDB(OrderDetail new_orderDetail, OrderDetail old_orderDetail)
        {
            DbConnection connection = null;
            DbCommand command = null;

            try
            {

                command = Database.GetStoredProcCommand(SPUpdateOrderDetail);
                Database.AddInParameter(command, "@orderId", DbType.Int32, new_orderDetail.OrderID);

                Database.AddInParameter(command, "@productID", DbType.Int32, new_orderDetail.ProductID);
                Database.AddInParameter(command, "@before_productID", DbType.Int32, old_orderDetail.ProductID);
                Database.AddInParameter(command, "@discount", DbType.Single, new_orderDetail.Discount);
                Database.AddInParameter(command, "@before_discount", DbType.Single, old_orderDetail.Discount);
                Database.AddInParameter(command, "@unitprice", DbType.Decimal, new_orderDetail.UnitPrice);
                Database.AddInParameter(command, "@before_unitprice", DbType.Decimal, old_orderDetail.UnitPrice);
                Database.AddInParameter(command, "@quantity", DbType.Int16, new_orderDetail.Quantity);
                Database.AddInParameter(command, "@before_quantity", DbType.Int16, old_orderDetail.Quantity);
                connection = Database.CreateConnection();
                command.Connection = connection;


                ETUtility.ExecuteNonQuery(Database, command);

            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return;

        }

        /// <summary>
        /// GetOrderDetailsByOrderIDDB
        /// </summary>
        /// <param name="orderID"></param>
        /// <returns>Array of OrderDetails</returns>
        internal OrderDetail[] GetOrderDetailsByOrderIDDB(int orderID)
        {
            ArrayList oOrderDetailArray = new ArrayList();
            DbCommand command = null;
            DbConnection connection = null;
            IDataReader reader = null;
            OrderDetail orderdetailtemp = null;


            try
            {
                command = Database.GetStoredProcCommand(SPGetOrderDeatilsByOrderID);
                Database.AddInParameter(command, "@orderID", DbType.Int32, orderID);

                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    oOrderDetailArray.Capacity = 100;
                    for (int i = 0; reader.Read(); i++)
                    {
                        MapReaderToOrderDetail(reader, orderdetailtemp = new OrderDetail());
                        oOrderDetailArray.Add(orderdetailtemp);
                    }
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return (OrderDetail[])oOrderDetailArray.ToArray(typeof(OrderDetail));
        }

        /// <summary>
        /// GetOrderDetailsByOrderIDDB
        /// </summary>
        /// <param name="orderID"></param>
        /// <returns>Array of OrderDetailProductDetail</returns>
        internal OrderDetailProductDetail[] GetOrderandProductDetailsByOrderIDDB(int orderID)
        {
            ArrayList oOrderDetailArray = new ArrayList();
            DbCommand command = null;
            DbConnection connection = null;
            IDataReader reader = null;
            OrderDetailProductDetail orderdetailtemp = null;


            try
            {
                command = Database.GetStoredProcCommand(SPGetOrderandProductDeatilsByOrderID);
                Database.AddInParameter(command, "@orderID", DbType.Int32, orderID);

                connection = Database.CreateConnection();
                command.Connection = connection;


                reader = ETUtility.ExecuteReader(Database, command);
                if (reader != null)
                {
                    oOrderDetailArray.Capacity = 20;
                    for (int i = 0; reader.Read(); i++)
                    {
                        MapReaderToOrderProductDetail(reader, orderdetailtemp = new OrderDetailProductDetail());
                        oOrderDetailArray.Add(orderdetailtemp);
                    }
                }
            }
            catch (Exception ex)
            {
                m_oLogUtility.TraceException(LogLevel.Error, ex.Message, ex);
                throw;
            }
            finally
            {
                if (reader != null && reader.IsClosed == false)
                {
                    reader.Close();
                }
                if (command != null)
                {
                    if (command.Connection != null)
                    {
                        command.Connection.Close();
                    }
                    command.Dispose();
                }
            }
            return (OrderDetailProductDetail[])oOrderDetailArray.ToArray(typeof(OrderDetailProductDetail));

        }
        /// <summary>
        /// MapReaderToOrder
        /// </summary>
        /// <param name="reader"></param>
        /// <param name="oOrder"></param>
        private void MapReaderToOrder(IDataReader reader, Order oOrder)
        {
            // Map Database Fields to Order Object
            oOrder.StaticDataId = reader.GetInt32(0);           // OrderID
            if (!reader.IsDBNull(1))
                oOrder.CustomerID = reader.GetString(1);        
            if (!reader.IsDBNull(2))
                oOrder.EmployeeId = reader.GetInt32(2);
            if (!reader.IsDBNull(3))
                oOrder.OrderDate = reader.GetDateTime(3);
            if (!reader.IsDBNull(4))
                oOrder.RequiredDate = reader.GetDateTime(4);
            if (!reader.IsDBNull(5))
                oOrder.ShippedDate = reader.GetDateTime(5);
            if (!reader.IsDBNull(6))
                oOrder.Freight = reader.GetDecimal(6);
            if (!reader.IsDBNull(7))
                oOrder.ShipName = reader.GetString(7);
            if (!reader.IsDBNull(8))
                oOrder.ShipAddress = reader.GetString(8);
            if (!reader.IsDBNull(9))
                oOrder.ShipCity = reader.GetString(9);
            if (!reader.IsDBNull(10))
                oOrder.ShipRegion = reader.GetString(10);
            if (!reader.IsDBNull(11))
                oOrder.ShipPostalCode = reader.GetString(11);
            if (!reader.IsDBNull(12))
                oOrder.ShipCountry = reader.GetString(12);

        }

        /// <summary>
        /// MapReaderToOrderProductDetail
        /// </summary>
        /// <param name="reader"></param>
        /// <param name="oOrderDetail"></param>
        private void MapReaderToOrderProductDetail(IDataReader reader, OrderDetailProductDetail oOrderProductDetail)
        {
             // Map Database Fields to OrderDetail Object
            if (!reader.IsDBNull(0))
                oOrderProductDetail.OrderID = reader.GetInt32(0);
            if (!reader.IsDBNull(1))
                oOrderProductDetail.ProductID = reader.GetInt32(1);
            if (!reader.IsDBNull(2))
                oOrderProductDetail.ProductName = reader.GetString(2);
            if (!reader.IsDBNull(3))
                oOrderProductDetail.UnitPrice = reader.GetDecimal(3);
            if (!reader.IsDBNull(4))
                oOrderProductDetail.Quantity = reader.GetInt16(4);
            if (!reader.IsDBNull(5))
                oOrderProductDetail.Discount = reader.GetFloat(5);
            if (!reader.IsDBNull(6))
                oOrderProductDetail.EffectivePrice = reader.GetDecimal(6);
            if (!reader.IsDBNull(7))
                oOrderProductDetail.TotalPrice = reader.GetDecimal(7);
            if (!reader.IsDBNull(8))
                oOrderProductDetail.QuantityPerUnit = reader.GetString(8);
            if (!reader.IsDBNull(9))
                oOrderProductDetail.UnitsInStock = reader.GetInt16(9);
            if (!reader.IsDBNull(10))
                oOrderProductDetail.CategoryName = reader.GetString(10);
            if (!reader.IsDBNull(11))
                oOrderProductDetail.Description = reader.GetString(11);

        }

        /// <summary>
        /// MapReaderToOrderProductDetail
        /// </summary>
        /// <param name="reader"></param>
        /// <param name="oOrderDetail"></param>
        private void MapReaderToOrderDetail(IDataReader reader, OrderDetail oOrderDetail)
        {
            // Map Database Fields to OrderDetail Object
            if (!reader.IsDBNull(0))
                oOrderDetail.OrderID = reader.GetInt32(0);
            if (!reader.IsDBNull(1))
                oOrderDetail.ProductID = reader.GetInt32(1);
            if (!reader.IsDBNull(2))
                oOrderDetail.UnitPrice = reader.GetDecimal(2);
            if (!reader.IsDBNull(3))
                oOrderDetail.Quantity = reader.GetInt16(3);
            if (!reader.IsDBNull(4))
                oOrderDetail.Discount = reader.GetFloat(4);

        }

    }
}
