﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Reflection;
using System.Security.Principal;
using System.Text;
using System.Threading;
//using System.Web.Security;
using Firma.Anwendung.Utility;

namespace Firma.Anwendung.Service
{
    /// <summary>
    /// BaseModule
    /// </summary>
    public abstract class BaseModule : IDisposable
    {
        /// <summary>
        /// LogUtility
        /// </summary>
        protected LogUtility m_oLogUtility = null;
        private bool m_fDisposed = false;

        /// <summary>
        /// Configuration
        /// </summary>
        protected Configuration m_oConfiguration = null;

        /// <summary>
        /// ServiceProxy
        /// </summary>
        protected readonly IServiceProxy m_oIServiceProxy = new ServiceProxy();

        /// <summary>
        /// WorkerThread
        /// </summary>
        protected Thread[] m_oWorkerThread = null;

        /// <summary>
        /// BaseModule
        /// </summary>
        public BaseModule()
        {
        }

        /// <summary>
        /// WorkerThread
        /// </summary>
        public Thread[] WorkerThread
        {
            get { return m_oWorkerThread; }
        }

        /// <summary>
        /// Dispose
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Dispose
        /// </summary>
        /// <param name="fDisposing"></param>
        protected virtual void Dispose(bool fDisposing)
        {
            if (!m_fDisposed)
            {
                m_oLogUtility.Dispose();
            }

            m_fDisposed = true;
        }

        /// <summary>
        /// ReadConfigFile
        /// </summary>
        /// <param name="oType"></param>
        /// <returns></returns>
        protected bool ReadConfigFile(Type oType)
        {
            bool fResult = false;

            try
            {
                Assembly oAssembly = Assembly.GetAssembly(oType);
                if (oAssembly != null)
                {
                    Uri oUri = new Uri(oAssembly.CodeBase);
                    if (File.Exists(oUri.LocalPath))
                    {
                        m_oConfiguration = ConfigurationManager.OpenExeConfiguration(oUri.LocalPath);
                        fResult = (m_oConfiguration != null) ? true : false;
                    }
                }
            }
            catch (Exception oException)
            {
                m_oLogUtility.TraceException(LogLevel.Error, "ReadConfigFile", oException);
            }

            return fResult;
        }

        /// <summary>
        /// Init
        /// </summary>
        /// <returns></returns>
        public abstract bool Init();

        /// <summary>
        /// Start
        /// </summary>
        /// <returns></returns>
        public abstract bool Start();

        /// <summary>
        /// Stop
        /// </summary>
        /// <returns></returns>
        public abstract bool Stop();
    }
}

