﻿using System;
using System.Diagnostics;
using System.IO;
using System.Web;

using log4net;
using log4net.Config;

namespace Firma.Anwendung.Utility
{
    /// <summary>
    /// LogUtility
    /// </summary>
    [Serializable]
    public class LogUtility : TraceBasic
    {
#if INTERNAL_LOGGING
        private static readonly LogInternalDebug m_oInternalLog = new LogInternalDebug();
#endif
        [NonSerialized]
        private ILog m_oLog4Net = null;

        private static bool m_fLoadConfiguration = false;

        /// <summary>
        /// LogUtility
        /// </summary>
        /// <param name="oType"></param>
        /// <param name="strConfigFile"></param>
        public LogUtility(Type oType, string strConfigFile)
            : base(oType.ToString())
        {
#if INTERNAL_LOGGING
            m_oInternalLog.Write("LogUtility(oType = '{0}', strConfigFile = '{1}')",
                oType.ToString(), strConfigFile);
#endif
            LoadConfigurationFromFile(strConfigFile);
            m_oLog4Net = LogManager.GetLogger(oType.ToString());
#if INTERNAL_LOGGING
            m_oInternalLog.WriteLogger(m_oLog4Net, "LogUtility(Type oType, string strConfigFile)");
#endif
        }

        /// <summary>
        /// LogUtility
        /// </summary>
        /// <param name="oType"></param>
        public LogUtility(Type oType)
            : base(oType.ToString())
        {
#if INTERNAL_LOGGING
            m_oInternalLog.Write("LogUtility(oType = '{0}')",
                oType.ToString());
#endif
            LoadConfiguration();
            m_oLog4Net = LogManager.GetLogger(oType.ToString());
#if INTERNAL_LOGGING
            m_oInternalLog.WriteLogger(m_oLog4Net, "LogUtility(Type oType)");
#endif
        }

        /// <summary>
        /// LogUtility
        /// </summary>
        /// <param name="strSourceName"></param>
        public LogUtility(string strSourceName)
            : base(strSourceName)
        {
#if INTERNAL_LOGGING
            m_oInternalLog.Write("LogUtility(strSourceName = '{0}')",
                    strSourceName);
#endif

            LoadConfiguration();
            m_oLog4Net = LogManager.GetLogger(strSourceName);
#if INTERNAL_LOGGING
            m_oInternalLog.WriteLogger(m_oLog4Net, "LogUtility(string strSourceName)");
#endif
        }

        private static void LoadConfigurationFromFile(string strConfigFile)
        {
#if INTERNAL_LOGGING
            m_oInternalLog.Write("LoadConfigurationFromFile(strConfigFile = '{0}')",
                strConfigFile);
#endif
            try
            {
                if (!m_fLoadConfiguration)
                {
#if INTERNAL_LOGGING
                    m_oInternalLog.Write("Configuration not loaded, try to load");
#endif
                    FileInfo oFileInfo = new FileInfo(strConfigFile);
                    if (File.Exists(oFileInfo.ToString()))
                    {
                        XmlConfigurator.ConfigureAndWatch(oFileInfo);
                        m_fLoadConfiguration = true;
#if INTERNAL_LOGGING
                        m_oInternalLog.Write("Configuration loaded");
#endif
                    }
                    else
                    {
#if INTERNAL_LOGGING
                        m_oInternalLog.Write("Config file '{0}' not found.", strConfigFile);
#endif
                    }
                }
            }
#pragma warning disable 168
            catch (Exception ex)
#pragma warning restore 168
            {
#if INTERNAL_LOGGING
                m_oInternalLog.Write("Exception in LoadConfiguration:\r\n{0}", ex.ToString()) ;
#endif
            }
#if INTERNAL_LOGGING
            m_oInternalLog.Write("---- LoadConfigurationFromFile() done");
#endif
        }

        private static void LoadConfiguration()
        {
#if INTERNAL_LOGGING
            m_oInternalLog.Write("LoadConfiguration()");
#endif
            try
            {
                if (!m_fLoadConfiguration)
                {
                    string strFilename;

                    // Check is WebApplication
                    if (HttpContext.Current != null)
                    {
                        strFilename = HttpContext.Current.Request.PhysicalApplicationPath;
                        strFilename += "TraceConfig";
#if INTERNAL_LOGGING
                       m_oInternalLog.Write("Loading via HttpContext.current, file '{0}'", strFilename);
#endif
                    }
                    else
                    {
                        strFilename = Process.GetCurrentProcess().MainModule.FileName.ToLower();
#if INTERNAL_LOGGING
                        m_oInternalLog.Write("Loading via current process '{0}'",strFilename);
#endif
                        int nPos = strFilename.LastIndexOf("vshost.exe");
                        if (nPos != -1)
                        {
                            strFilename = strFilename.Substring(0, nPos) + "exe";
#if INTERNAL_LOGGING
                            m_oInternalLog.Write("Removed 'vshost.', resulting in '{0}'", strFilename);
#endif
                        }
                    }

                    FileInfo oFileInfo = new FileInfo(strFilename + ".log4net");
                    if (File.Exists(oFileInfo.ToString()))
                    {
#if INTERNAL_LOGGING
                        m_oInternalLog.Write("Load config from '{0}'", oFileInfo.ToString());
#endif
                        XmlConfigurator.ConfigureAndWatch(oFileInfo);
                        m_fLoadConfiguration = true;
                    }
                    else
                    {
#if INTERNAL_LOGGING
                        m_oInternalLog.Write("Config file '{0}' not found.", oFileInfo.ToString());
#endif
                    }
                }
            }
#pragma warning disable 168
            catch (Exception ex)
#pragma warning restore 168
            {
#if INTERNAL_LOGGING
                m_oInternalLog.Write("Exception in LoadConfiguration:\r\n{0}", ex.ToString());
#endif
            }
#if INTERNAL_LOGGING
            m_oInternalLog.Write("---- LoadConfiguration() done");
#endif
        }

        /// <summary>
        /// Dispose
        /// </summary>
        /// <param name="disposing"></param>
        protected override void Dispose(bool disposing)
        {
#if INTERNAL_LOGGING
            m_oInternalLog.Write("LogUtility.Dispose({0})", disposing);
#endif
            if (!m_fDisposed)
            {
                if (m_oLog4Net != null)
                {
#if INTERNAL_LOGGING
                    m_oInternalLog.Write("Disposing m_oLog4Net '{0}'", m_oLog4Net.Logger.Name);
#endif
                    m_oLog4Net.DebugFormat("Dispose LogUtility");
                    m_oLog4Net = null;
                }
            }

            m_fDisposed = true;
        }

        /// <summary>
        /// IsDebugEnabled
        /// </summary>
        public bool IsDebugEnabled
        {
            get { return m_oLog4Net.IsDebugEnabled; }
        }

        /// <summary>
        /// IsInfoEnabled
        /// </summary>
        public bool IsInfoEnabled
        {
            get { return m_oLog4Net.IsInfoEnabled; }
        }

        /// <summary>
        /// IsWarnEnabled
        /// </summary>
        public bool IsWarnEnabled
        {
            get { return m_oLog4Net.IsWarnEnabled; }
        }

        /// <summary>
        /// IsErrorEnabled
        /// </summary>
        public bool IsErrorEnabled
        {
            get { return m_oLog4Net.IsErrorEnabled; }
        }

        /// <summary>
        /// IsFatalEnabled
        /// </summary>
        public bool IsFatalEnabled
        {
            get { return m_oLog4Net.IsFatalEnabled; }
        }

        /// <summary>
        /// TraceLog
        /// </summary>
        /// <param name="eTraceLevel"></param>
        /// <param name="strFormat"></param>
        /// <param name="oArgs"></param>
        public override void TraceLog(LogLevel eTraceLevel, string strFormat, params object[] oArgs)
        {
            try
            {
                EnsureLogExists();
                if (null != m_oLog4Net)
                {
                    switch (eTraceLevel)
                    {
                        case LogLevel.Debug:
                            m_oLog4Net.DebugFormat(strFormat, oArgs);
                            break;
                        case LogLevel.Info:
                            m_oLog4Net.InfoFormat(strFormat, oArgs);
                            break;
                        case LogLevel.Warning:
                            m_oLog4Net.WarnFormat(strFormat, oArgs);
                            break;
                        case LogLevel.Error:
                            m_oLog4Net.ErrorFormat(strFormat, oArgs);
                            break;
                        case LogLevel.Fatal:
                            m_oLog4Net.FatalFormat(strFormat, oArgs);
                            break;
                    }
                }
            }
#pragma warning disable 168
            catch (Exception ex)
#pragma warning restore 168
            {
#if INTERNAL_LOGGING
                m_oInternalLog.Write("Exception writing to logger '{0}'\r\n{1}",
                    m_oLog4Net.Logger.Name, ex.ToString());
#endif
            }
        }

        /// <summary>
        /// EnsureLogExists
        /// </summary>
        private void EnsureLogExists()
        {
            if (null == m_oLog4Net)
            {
                m_oLog4Net = LogManager.GetLogger(m_strSourceName);
#if INTERNAL_LOGGING
                m_oInternalLog.Write("Get logger '{0}' for source '{0}'",
                    m_oLog4Net.Logger.Name, m_strSourceName);
                m_oInternalLog.WriteLogger(m_oLog4Net, "EnsureLogExists()");
#endif
            }
        }

        /// <summary>
        /// TraceException
        /// </summary>
        /// <param name="eTraceLevel"></param>
        /// <param name="strComment"></param>
        /// <param name="oException"></param>
        public override void TraceException(LogLevel eTraceLevel, string strComment, Exception oException)
        {
            try
            {
                EnsureLogExists();
                if (null != m_oLog4Net)
                {
                    switch (eTraceLevel)
                    {
                        case LogLevel.Debug:
                            m_oLog4Net.Debug(strComment, oException);
                            break;
                        case LogLevel.Info:
                            m_oLog4Net.Info(strComment, oException);
                            break;
                        case LogLevel.Warning:
                            m_oLog4Net.Warn(strComment, oException);
                            break;
                        case LogLevel.Error:
                            m_oLog4Net.Error(strComment, oException);
                            break;
                        case LogLevel.Fatal:
                            m_oLog4Net.Fatal(strComment, oException);
                            break;
                    }
                }
            }
#pragma warning disable 168
            catch (Exception ex)
#pragma warning restore 168
            {
#if INTERNAL_LOGGING
               m_oInternalLog.Write("Exception writing to logger from TraceException() '{0}'\r\n{1}",
                    m_oLog4Net.Logger.Name, ex.ToString());
#endif
            }
        }
    }
}
