﻿using System;
using System.Diagnostics;
using System.IO;

using log4net;

namespace Firma.Anwendung.Utility
{
    /// <summary>
    /// LogInternalDebug
    /// </summary>
    public class LogInternalDebug : IDisposable
    {
        private const string _dstDir = @"C:\Net4Log";

        private StreamWriter _logWr = null;

        /// <summary>
        /// LogInternalDebug
        /// </summary>
        public LogInternalDebug()
        {
            Directory.CreateDirectory(_dstDir);
            Guid guid = Guid.NewGuid();
            string dstfile = Path.Combine(_dstDir, "InternalLog_" + guid + ".log");
            try
            {
                _logWr = new StreamWriter(dstfile, false);
                _logWr.AutoFlush = true;
                Write("Instance {0}", guid);
                WriteStack();
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// WriteLogger
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="caller"></param>
        public void WriteLogger(ILog logger, string caller)
        {
            string level = "Unknown";
            if (logger.IsDebugEnabled)
                level = "DEBUG";
            else if (logger.IsInfoEnabled)
                level = "INFO";
            else if (logger.IsWarnEnabled)
                level = "WARN";
            else if (logger.IsErrorEnabled)
                level = "ERROR";
            else if (logger.IsFatalEnabled)
                level = "FATAL";
            Write("{0} is using logger '{1}' with level {2}", caller, logger.Logger.Name, level);
        }

        /// <summary>
        /// Write
        /// </summary>
        /// <param name="fmt"></param>
        /// <param name="args"></param>
        public void Write(string fmt, params object[] args)
        {
            if (_logWr != null)
            {
                lock (_logWr)
                {
                    _logWr.Write("{0:HH:mm:ss.fff} ", DateTime.Now);
                    _logWr.WriteLine(fmt, args);
                }
            }
        }

        /// <summary>
        /// Dispose
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Dispose
        /// </summary>
        /// <param name="disposing"></param>
        private void Dispose(bool disposing)
        {
            Write("Dispose ({0})", disposing);
            if (_logWr != null)
            {
                _logWr.Close();
                _logWr = null;
            }
        }

        /// <summary>
        /// WriteStack
        /// </summary>
        private void WriteStack()
        {
            if (_logWr != null)
            {
                StackTrace stack = new StackTrace(true);
                Write("Call Stack:\r\n{0}", stack);
            }
        }
    }
}
