﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Firma.Anwendung.Utility
{
    /// <summary>
    /// TimeLogger
    /// </summary>
    public class TimeLogger
    {
#if TIMELOGGER
        // Lock object
        private static readonly object m_oCreateTimeLoggerLock = new object();

        /// <summary>
        /// The Logger.
        /// </summary>
        private LogUtility m_oLogUtility = null;

        /// <summary>
        /// The Timeloggers
        /// </summary>
        private static Dictionary<string, TimeLogger> m_oTimeLoggers = null;

        private long m_oStart = 0;
        private long m_oStop = 0;
#endif
        // Singletons
        private TimeLogger()
        {
        }

        /// <summary>
        /// Start the Timer
        /// </summary>
        public void Start()
        {
#if TIMELOGGER
            m_oStart = DateTime.Now.Ticks;
#endif
        }
        /// <summary>
        /// Stop the Timer
        /// </summary>
        public void Stop()
        {
#if TIMELOGGER
            m_oStop = DateTime.Now.Ticks;
#endif
        }

        /// <summary>
        /// Log the elapsed time
        /// </summary>
        /// <param name="message">An additional message, e.g. the method name</param>
        public void Log(String message)
        {
#if TIMELOGGER
            long elapsedTime = m_oStop - m_oStart;
            m_oLogUtility.TraceLog(GuD.ICMS.Utility.LogLevel.Info, message + " Elapsed time (ms) :" + elapsedTime / 10000);
#endif

        }
        /// <summary>
        /// Get the on and only instance
        /// </summary>
        /// <returns></returns>
        public static TimeLogger Instance(Type type)
        {
#if TIMELOGGER
            return Instance(type.ToString());
#else
            return new TimeLogger();
#endif
        }

        /// <summary>
        /// Get the on and only instance
        /// </summary>
        /// <returns></returns>
        public static TimeLogger Instance(String type)
        {
#if TIMELOGGER
            String strIdentifier = type.ToString();
            lock (m_oCreateTimeLoggerLock)
            {
                // Use 'Lazy initialization'
                if (m_oTimeLoggers == null)
                {
                    m_oTimeLoggers = new Dictionary<string, TimeLogger>();
                }
                if (!m_oTimeLoggers.ContainsKey(strIdentifier))
                {
                    try
                    {
                        m_oTimeLoggers.Add(strIdentifier, new TimeLogger());
                    }
                    catch (Exception)
                    {
                        //identifier was already there 
                    }

                    m_oTimeLoggers[strIdentifier].m_oLogUtility = new LogUtility(typeof(TimeLogger) + "." + strIdentifier);
                }
            }

            return m_oTimeLoggers[strIdentifier];
#else
            return new TimeLogger();
#endif
        }
    }
}

