﻿using System;
using System.Collections.Generic;
using System.Text;

using TraceServices;

namespace Firma.Anwendung.Utility
{
    /// <summary>
    /// LogUtility
    /// </summary>
    [Serializable]
    public class LogUtility : TraceBasic
    {
        /// <summary>
        /// LogUtility
        /// </summary>
        /// <param name="strSourceName"></param>
        public LogUtility(string strSourceName)
            : base(strSourceName)
        {
        }

        /// <summary>
        /// Dispose
        /// </summary>
        /// <param name="disposing"></param>
        protected override void Dispose(bool disposing)
        {
            if (!m_fDisposed)
            {
                if (m_strSourceName != string.Empty)
                {
                    LocalTrace oLocalTrace = (LocalTrace)TraceManager.LocalTracePool[m_strSourceName];
                    if (oLocalTrace != null)
                    {
                        oLocalTrace.UnregisterSource(false);
                        oLocalTrace = null;

                        m_strSourceName = string.Empty;
                    }
                }
            }

            m_fDisposed = true;
        }

        /// <summary>
        /// IsDebugEnabled
        /// </summary>
        public override bool IsDebugEnabled
        {
            get
            {
                return TraceManager.IsTraceEnabled(m_strSourceName, TraceServices.LogLevel.All);
            }
        }

        /// <summary>
        /// IsInfoEnabled
        /// </summary>
        public bool IsInfoEnabled
        {
            get
            {
                return TraceManager.IsTraceEnabled(m_strSourceName, TraceServices.LogLevel.Information);
            }
        }

        /// <summary>
        /// IsWarnEnabled
        /// </summary>
        public bool IsWarnEnabled
        {
            get
            {
                return TraceManager.IsTraceEnabled(m_strSourceName, TraceServices.LogLevel.Warning);
            }
        }

        /// <summary>
        /// IsErrorEnabled
        /// </summary>
        public bool IsErrorEnabled
        {
            get
            {
                return TraceManager.IsTraceEnabled(m_strSourceName, TraceServices.LogLevel.Error);
            }
        }

        /// <summary>
        /// IsFatalEnabled
        /// </summary>
        public bool IsFatalEnabled
        {
            get
            {
                return TraceManager.IsTraceEnabled(m_strSourceName, TraceServices.LogLevel.Error);
            }
        }

        /// <summary>
        /// MatchLogLevel
        /// </summary>
        /// <param name="eLogLevel"></param>
        /// <returns></returns>
        private TraceServices.LogLevel MatchLogLevel(LogLevel eLogLevel)
        {
            TraceServices.LogLevel eLogServiceLevel = TraceServices.LogLevel.Error;

            switch (eLogLevel)
            {
                case LogLevel.Debug:
                    eLogServiceLevel = TraceServices.LogLevel.All;
                    break;
                case LogLevel.Info:
                    eLogServiceLevel = TraceServices.LogLevel.Information;
                    break;
                case LogLevel.Warning:
                    eLogServiceLevel = TraceServices.LogLevel.Warning;
                    break;
                case LogLevel.Error:
                case LogLevel.Fatal:
                    eLogServiceLevel = TraceServices.LogLevel.Error;
                    break;
            }

            return eLogServiceLevel;
        }

        /// <summary>
        /// TraceLog
        /// </summary>
        /// <param name="eLogLevel"></param>
        /// <param name="strFormat"></param>
        /// <param name="oArgs"></param>
        public override void TraceLog(LogLevel eLogLevel, string strFormat, params object[] oArgs)
        {
            try
            {
                LocalTrace oLocalTrace = (LocalTrace)TraceManager.LocalTracePool[m_strSourceName];
                TraceServices.LogLevel eTraceServiceLevel = MatchLogLevel(eLogLevel);
                if ((oLocalTrace != null) && (TraceManager.IsTraceEnabled(m_strSourceName, eTraceServiceLevel)))
                {
                    oLocalTrace.TraceL(eTraceServiceLevel, strFormat, oArgs);
                }
            }
            catch
            { }
        }

        /// <summary>
        /// TraceException
        /// </summary>
        /// <param name="eLogLevel"></param>
        /// <param name="strComment"></param>
        /// <param name="oException"></param>
        public override void TraceException(LogLevel eLogLevel, string strComment, Exception oException)
        {
            try
            {
                LocalTrace oLocalTrace = (LocalTrace)TraceManager.LocalTracePool[m_strSourceName];
                TraceServices.LogLevel eTraceServiceLevel = MatchLogLevel(eLogLevel);
                if ((oLocalTrace != null) && (TraceManager.IsTraceEnabled(m_strSourceName, eTraceServiceLevel)))
                {
                    oLocalTrace.TraceE(eTraceServiceLevel, strComment, oException);
                }
            }
            catch
            { }
        }
    }
}
