﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MyServiceAgent.NWCustomerService;
using MyServiceAgent.NWOrderService;
using MyServiceAgent.NWProductService;


namespace ETWPFTestClient
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Customer CurrentCustomerOrigin;
        private Customer CurrentCustomerNew;
        private Customer[] CustomerFound = null;

        private Order CurrentOrderOrigin;
        private Order CurrentOrderNew;
        private Order[] OrderFound = null;

        private OrderDetailProductDetail CurrentOrderDetailProductDetailOrigin;
        private OrderDetailProductDetail CurrentOrderDetailProductDetailNew;
        private OrderDetailProductDetail[] OrderDetailsProductDetailsFound;


        public MainWindow()
        {
            InitializeComponent();
            txtCustomerSearch.Focus();
        }


        private void bSearchCustomer_Click(object sender, RoutedEventArgs e)
        {
            this.Cursor = Cursors.Wait;
            CustomerFound = null;
            if (txtCustomerSearch.Text != String.Empty)
            {
                CustomerServiceClient csc = new CustomerServiceClient();
                CustomerFound = csc.GetCustomerArrayForName(txtCustomerSearch.Text);
                if (CustomerFound.Length == 0)
                {
                    txtCustSearchMsg.Text = "No Customer found matching your Search String!";
                    CustomerListRO.Visibility = Visibility.Collapsed;
                    OrderGrid.Visibility = Visibility.Collapsed;
                    OrderDetailsGrid.Visibility = Visibility.Collapsed;
                    txtCustSearchMsg.Visibility = Visibility.Visible;
                }
                else if (CustomerFound.Length >= 1)
                {
                    CustomerListRO.DataContext = CustomerFound;
                    CustomerListRO.Visibility = Visibility.Visible;

                }
                //else if (CustomerFound.Length == 1)
                //{
                //    CustomerListRO.DataContext = CustomerFound;
                //    CustomerListRO.Visibility = Visibility.Visible;

                //}
            }
            else
            {
                txtCustSearchMsg.Text = "Please provide a valid Search String!";
                CustomerListRO.Visibility = Visibility.Collapsed;
                txtCustSearchMsg.Visibility = Visibility.Visible;

            }
            
            this.Cursor = Cursors.Arrow;
        }

        private void txtCustomerSearch_TextChanged(object sender, TextChangedEventArgs e)
        {
            txtCustSearchMsg.Visibility = Visibility.Hidden;
            txtCustSearchMsg.Text = String.Empty;
        }

        private void EditCustMenu_Click(object sender, RoutedEventArgs e)
        {
            CurrentCustomerOrigin = ((sender as MenuItem).Tag as Customer);
            CurrentCustomerNew = new Customer();

            UpdateCustomer(CurrentCustomerNew, CurrentCustomerOrigin);

        }

        private void CustomerListRO_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            ListView lvi = sender as ListView;
            CurrentCustomerOrigin = lvi.SelectedItem as Customer;

            CurrentCustomerNew = new Customer();
            UpdateCustomer(CurrentCustomerNew, CurrentCustomerOrigin);
        }


        void DeleteCustMenu_Click(object sender, RoutedEventArgs e)
        {
            CurrentCustomerOrigin = ((sender as MenuItem).Tag as Customer);
            CustomerServiceClient csc = new CustomerServiceClient();
            try
            {
                csc.DeleteCustomer(CurrentCustomerOrigin.StaticDataId);
                CustomerListRO.DataContext = null;
                CustomerFound = null;
                CustomerFound = csc.GetCustomerArrayForName(txtCustomerSearch.Text);
                CustomerListRO.DataContext = CustomerFound;
            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.Message, "Delete Custoomer Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void bCreateCustomer_Click(object sender, RoutedEventArgs e)
        {
            CurrentCustomerNew = new Customer();
            NewCustomerWindow ncw = new NewCustomerWindow(CurrentCustomerNew);
                        ncw.WindowStyle = WindowStyle.SingleBorderWindow;
            Nullable<Boolean> result = false;
            result = ncw.ShowDialog();
            if (result == true)
            {
                CustomerServiceClient csc = new CustomerServiceClient();
                try
                {
                    csc.CreateCustomer(CurrentCustomerNew);
                    CustomerListRO.DataContext = null;
                    CustomerFound = null;
                    if (txtCustomerSearch.Text != String.Empty)
                    {
                        RoutedEventArgs ev = new RoutedEventArgs();
                        Object s = new Object();
                        bSearchCustomer_Click(s, ev);
                    }


                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message, "Update Customer Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }

        }

        private void MenuCustOrders_Click(object sender, RoutedEventArgs e)
        {
            this.Cursor = Cursors.Wait;

            int selection = 0;
            Customer CurrCust = (sender as MenuItem).Tag as Customer;
            OrderServiceClient osc = new OrderServiceClient();
            OrderFound = osc.GetOrdersforCustomer(CurrCust.StaticDataId);
            OrderListRO.DataContext = OrderFound;
            OrderGrid.Visibility = Visibility.Visible;
            foreach (Customer c in CustomerListRO.Items)
            {
                if (CurrCust.StaticDataId == c.StaticDataId)
                {
                    break;
                }
                else
                {
                    selection++;
                }
            }
            CustomerListRO.SelectedIndex = selection;

            this.Cursor = Cursors.Arrow;


        }

        private void MenuCustNewOrder_Click(object sender, RoutedEventArgs e)
        {
            Random RandomEmpID = new Random();
            
            Customer CurrCust = (sender as MenuItem).Tag as Customer;
            Order NewOrder = new Order();
            NewOrder.OrderDate = DateTime.Today;
            NewOrder.ShippedDate = DateTime.Today.AddDays(7);
            NewOrder.RequiredDate = DateTime.Today.AddDays(21);
            NewOrder.CustomerID = CurrCust.StaticDataId;
            NewOrder.ShipAddress = CurrCust.Address;
            NewOrder.ShipCity = CurrCust.City;
            NewOrder.ShipCountry = CurrCust.Country;
            NewOrder.ShipName = CurrCust.CompanyName;
            NewOrder.ShipPostalCode = CurrCust.PostalCode;
            // NewOrder.ShipRegion = "";
            NewOrder.EmployeeId = RandomEmpID.Next(1, 9);

                        NewOrderWindow now = new NewOrderWindow(NewOrder);
                        now.WindowStyle = WindowStyle.SingleBorderWindow;
            Nullable<Boolean> result = false;
            result = now.ShowDialog();
            if (result == true)
            {
                OrderServiceClient osc = new OrderServiceClient();
                try
                {
                    osc.CreateOrder(NewOrder);
                    OrderListRO.DataContext = null;
                    OrderFound = null;
                    OrderFound = osc.GetOrdersforCustomer(CurrCust.StaticDataId);
                    OrderListRO.DataContext = OrderFound;
                    OrderGrid.Visibility = Visibility.Visible;

                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message, "Update Customer Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }

            

        }


        private void CustomerListRO_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.Cursor = Cursors.Wait;

            ListView lv = (sender as ListView);
            if (lv.SelectedIndex != -1)
            {
                Customer CurrCust = lv.SelectedItem as Customer;
                OrderServiceClient osc = new OrderServiceClient();
                OrderFound = osc.GetOrdersforCustomer(CurrCust.StaticDataId);
                OrderListRO.DataContext = OrderFound;
                OrderGrid.Visibility = Visibility.Visible;
            }
            else
            {
                OrderGrid.Visibility = Visibility.Collapsed;
                OrderListRO.DataContext = null;
            }

            this.Cursor = Cursors.Arrow;

        }

        private void OrderListRO_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            ListView lvi = sender as ListView;
            CurrentOrderOrigin = lvi.SelectedItem as Order;

            CurrentOrderNew = new Order();
            UpdateOrder(CurrentOrderNew, CurrentOrderOrigin);


        }

        internal void UpdateCustomer(Customer CurrentCustomerNew, Customer CurrentCustomerOrigin)
        {
            CurrentCustomerNew.StaticDataId = CurrentCustomerOrigin.StaticDataId;
            CurrentCustomerNew.CompanyName = CurrentCustomerOrigin.CompanyName;
            CurrentCustomerNew.ContactTitle = CurrentCustomerOrigin.ContactTitle;
            CurrentCustomerNew.ContactName = CurrentCustomerOrigin.ContactName;
            CurrentCustomerNew.Address = CurrentCustomerOrigin.Address;
            CurrentCustomerNew.City = CurrentCustomerOrigin.City;
            CurrentCustomerNew.Phone = CurrentCustomerOrigin.Phone;
            CurrentCustomerNew.Country = CurrentCustomerOrigin.Country;
            CurrentCustomerNew.Fax = CurrentCustomerOrigin.Fax;
            CurrentCustomerNew.PostalCode = CurrentCustomerOrigin.PostalCode;

            UpdateCustomerWindow ucw = new UpdateCustomerWindow(CurrentCustomerNew);
            ucw.WindowStyle = WindowStyle.SingleBorderWindow;
            Nullable<Boolean> result = false;
            result = ucw.ShowDialog();
            if ((result == true) && !(CompareCustomer(CurrentCustomerNew, CurrentCustomerOrigin)))
            {
                CustomerServiceClient csc = new CustomerServiceClient();
                try
                {
                    csc.UpdateCustomer(CurrentCustomerNew, CurrentCustomerOrigin);
                    CustomerListRO.DataContext = null;
                    CustomerFound = null;
                    CustomerFound = csc.GetCustomerArrayForName(txtCustomerSearch.Text);
                    CustomerListRO.DataContext = CustomerFound;


                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message, "Update Customer Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }

            }

        }

        internal void UpdateOrder(Order CurrentOrderNew, Order CurrentOrderOrigin)
        {

            CurrentOrderNew.StaticDataId = CurrentOrderOrigin.StaticDataId;
            CurrentOrderNew.EmployeeId = CurrentOrderOrigin.EmployeeId;
            CurrentOrderNew.CustomerID = CurrentOrderOrigin.CustomerID;
            CurrentOrderNew.Freight = CurrentOrderOrigin.Freight;
            CurrentOrderNew.OrderDate = CurrentOrderOrigin.OrderDate;
            CurrentOrderNew.RequiredDate = CurrentOrderOrigin.RequiredDate;
            CurrentOrderNew.ShipAddress = CurrentOrderOrigin.ShipAddress;
            CurrentOrderNew.ShipCity = CurrentOrderOrigin.ShipCity;
            CurrentOrderNew.ShipCountry = CurrentOrderOrigin.ShipCountry;
            CurrentOrderNew.ShipName = CurrentOrderOrigin.ShipName;
            CurrentOrderNew.ShippedDate = CurrentOrderOrigin.ShippedDate;
            CurrentOrderNew.ShipPostalCode = CurrentOrderOrigin.ShipPostalCode;


            UpdateOrderWindow uow = new UpdateOrderWindow(CurrentOrderNew);
            uow.WindowStyle = WindowStyle.SingleBorderWindow;
            Nullable<Boolean> result = false;
            result = uow.ShowDialog();
            if ((result == true) && !(CompareOrder(CurrentOrderNew, CurrentOrderOrigin)))
            {
                OrderServiceClient osc = new OrderServiceClient();
                try
                {
                    osc.UpdateOrder(CurrentOrderNew, CurrentOrderOrigin);
                    OrderListRO.DataContext = null;
                    OrderFound = null;
                    OrderFound = osc.GetOrdersforCustomer(CurrentOrderOrigin.CustomerID);
                    OrderListRO.DataContext = OrderFound;


                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message, "Update Order Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }

            }

        }

        internal void UpdateOrderDetail(OrderDetailProductDetail CurrentOrderDetailNew, OrderDetailProductDetail CurrentOrderDetailOrigin)
        {
            CurrentOrderDetailNew.CategoryName = CurrentOrderDetailOrigin.CategoryName;
            CurrentOrderDetailNew.Description = CurrentOrderDetailOrigin.Description;
            CurrentOrderDetailNew.Discount = CurrentOrderDetailOrigin.Discount;
            CurrentOrderDetailNew.EffectivePrice = CurrentOrderDetailOrigin.EffectivePrice;
            CurrentOrderDetailNew.ProductID = CurrentOrderDetailOrigin.ProductID;
            CurrentOrderDetailNew.ProductName = CurrentOrderDetailOrigin.ProductName;
            CurrentOrderDetailNew.Quantity = CurrentOrderDetailOrigin.Quantity;
            CurrentOrderDetailNew.QuantityPerUnit = CurrentOrderDetailOrigin.QuantityPerUnit;
            CurrentOrderDetailNew.TotalPrice = CurrentOrderDetailOrigin.TotalPrice;
            CurrentOrderDetailNew.UnitPrice = CurrentOrderDetailOrigin.UnitPrice;
            CurrentOrderDetailNew.UnitsInStock = CurrentOrderDetailOrigin.UnitsInStock;
            CurrentOrderDetailNew.OrderID = CurrentOrderDetailOrigin.OrderID;

            UpdateOrderDetailWindow uodw = new UpdateOrderDetailWindow(CurrentOrderDetailNew);
            uodw.WindowStyle = WindowStyle.SingleBorderWindow;
            Nullable<Boolean> result = false;
            result = uodw.ShowDialog();
            if ((result == true) && !(CompareOrderDetail(CurrentOrderDetailNew, CurrentOrderDetailOrigin)))
            {
                OrderDetail nod = new OrderDetail();
                nod.ProductID = CurrentOrderDetailNew.ProductID;
                nod.OrderID = CurrentOrderDetailNew.OrderID;
                nod.Discount = CurrentOrderDetailNew.Discount;
                nod.Quantity = CurrentOrderDetailNew.Quantity;
                nod.UnitPrice = CurrentOrderDetailNew.UnitPrice;

                OrderDetail ood = new OrderDetail();
                ood.ProductID = CurrentOrderDetailOrigin.ProductID;
                ood.OrderID = CurrentOrderDetailOrigin.OrderID;
                ood.Discount = CurrentOrderDetailOrigin.Discount;
                ood.UnitPrice = CurrentOrderDetailOrigin.UnitPrice;
                ood.Quantity = CurrentOrderDetailOrigin.Quantity;


                OrderServiceClient osc = new OrderServiceClient();
                try
                {
                    osc.UpdateOrderDetail(nod, ood);
                    OrderDetailsListRO.DataContext = null;
                    OrderDetailsProductDetailsFound = null;
                    OrderDetailsProductDetailsFound = osc.GetOrderandProductDetailsByOrderID(ood.OrderID);
                    OrderDetailsListRO.DataContext = OrderDetailsProductDetailsFound;


                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message, "Update Order Detail Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }

            }

        }

        internal void NewOrderDetail(OrderDetailProductDetail CurrentOrderDetailNew)
        {
            NewOrderDetailWindow nodw = new NewOrderDetailWindow(CurrentOrderDetailNew);
            nodw.WindowStyle = WindowStyle.SingleBorderWindow;
            Nullable<Boolean> result = false;
            result = nodw.ShowDialog();
            if (result == true)
            {

                OrderDetail nod = new OrderDetail();
                nod.ProductID = CurrentOrderDetailNew.ProductID;
                nod.OrderID = CurrentOrderDetailNew.OrderID;
                nod.Discount = CurrentOrderDetailNew.Discount;
                nod.Quantity = CurrentOrderDetailNew.Quantity;
                nod.UnitPrice = CurrentOrderDetailNew.UnitPrice;


                OrderServiceClient osc = new OrderServiceClient();
                try
                {
                    osc.CreateOrderDetail(nod);
                    OrderDetailsListRO.DataContext = null;
                    OrderDetailsProductDetailsFound = null;
                    OrderDetailsProductDetailsFound = osc.GetOrderandProductDetailsByOrderID(nod.OrderID);
                    OrderDetailsListRO.DataContext = OrderDetailsProductDetailsFound;


                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message, "New Order Detail Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }

        }


        internal bool CompareCustomer(Customer c1, Customer c2)
        {
            if (c1.CompanyName != c2.CompanyName)
            {
               return false;
            }
            if (c1.ContactName != c2.ContactName)
            {
                return false;
            }
            if (c1.ContactTitle != c2.ContactTitle)
            {
                return false;
            }
            if (c1.Country != c2.Country)
            {
                return false;
            }
            if (c1.Address != c2.Address)
            {
                return false;
            }
            if (c1.City != c2.City)
            {
                return false;
            }
            if (c1.Fax != c2.Fax)
            {
                return false;
            }
            if (c1.Phone != c2.Phone)
            {
                return false;
            }
            if (c1.PostalCode != c2.PostalCode)
            {
                return false;
            }

            return true;
        }

        internal bool CompareOrder( Order o1, Order o2)
        {
            if ( o1.StaticDataId != o2.StaticDataId)
            {
                return false;
            }
            if ( o1.CustomerID != o2.CustomerID)
            {
                return false;
            }
            if ( o1.Freight != o2.Freight)
            {
                return false;
            }
            if ( o1.OrderDate != o2.OrderDate)
            {
                return false;
            }
            if ( o1.RequiredDate != o2.RequiredDate)
            {
                return false;
            }
            if ( o1.ShipAddress != o2.ShipAddress)
            {
                return false;
            }
            if ( o1.ShipCity != o2.ShipCity)
            {
                return false;
            }
            if ( o1.ShipCountry != o2.ShipCountry)
            {
                return false;
            }
            if ( o1.ShipName != o2.ShipName)
            {
                return false;
            }
            if (o1.ShippedDate != o2.ShippedDate)
            {
                return false;
            }
            if (o1.ShipPostalCode != o2.ShipPostalCode)
            {
                return false;
            }

            return true;
        }

        internal bool CompareOrderDetail(OrderDetailProductDetail CurrentOrderDetailNew, OrderDetailProductDetail CurrentOrderDetailOrigin)
        {
            if (CurrentOrderDetailNew.ProductID != CurrentOrderDetailOrigin.ProductID)
            {
                return false;
            }
            if (CurrentOrderDetailNew.Quantity != CurrentOrderDetailOrigin.Quantity)
            {
                return false;
            }
            if (CurrentOrderDetailNew.Discount != CurrentOrderDetailOrigin.Discount)
            {
                return false;
            }
            if (CurrentOrderDetailNew.UnitPrice != CurrentOrderDetailOrigin.UnitPrice)
            {
                return false;
            }
            return true;
        }

        private void MenuOrderEdit_Click(object sender, RoutedEventArgs e)
        {
            CurrentOrderOrigin = ((sender as MenuItem).Tag as Order);
            CurrentOrderNew = new Order();

            UpdateOrder(CurrentOrderNew, CurrentOrderOrigin);


        }

        private void MenuOrderOrderDetails_Click(object sender, RoutedEventArgs e)
        {
            this.Cursor = Cursors.Wait;

            Order CurrOrder = (sender as MenuItem).Tag as Order;
            int selection = 0;
            OrderServiceClient osc = new OrderServiceClient();
            OrderDetailsProductDetailsFound = osc.GetOrderandProductDetailsByOrderID(CurrOrder.StaticDataId);
            OrderDetailsListRO.DataContext = OrderDetailsProductDetailsFound;
            OrderDetailsGrid.Visibility = Visibility.Visible;
            foreach (Order o in OrderListRO.Items)
            {
                if (CurrOrder.StaticDataId == o.StaticDataId)
                {
                    break;
                }
                else
                {
                    selection++;
                }
            }
            OrderListRO.SelectedIndex = selection;

            this.Cursor = Cursors.Arrow;

        }

        private void MenuOrderNewOrder_Click(object sender, RoutedEventArgs e)
        {
            Random RandomEmpID = new Random();

            Order CurrOrder = (sender as MenuItem).Tag as Order;
            Order NewOrder = new Order();
            NewOrder.OrderDate = DateTime.Today;
            NewOrder.ShippedDate = DateTime.Today.AddDays(7);
            NewOrder.RequiredDate = DateTime.Today.AddDays(21);
            NewOrder.CustomerID = CurrOrder.CustomerID;
            NewOrder.ShipAddress = CurrOrder.ShipAddress;
            NewOrder.ShipCity = CurrOrder.ShipCity;
            NewOrder.ShipCountry = CurrOrder.ShipCountry;
            NewOrder.ShipName = CurrOrder.ShipName;
            NewOrder.ShipPostalCode = CurrOrder.ShipPostalCode;
            // NewOrder.ShipRegion = "";
            NewOrder.EmployeeId = RandomEmpID.Next(1, 9);

            NewOrderWindow now = new NewOrderWindow(NewOrder);
            now.WindowStyle = WindowStyle.SingleBorderWindow;
            Nullable<Boolean> result = false;
            result = now.ShowDialog();
            if (result == true)
            {
                OrderServiceClient osc = new OrderServiceClient();
                try
                {
                    osc.CreateOrder(NewOrder);
                    OrderListRO.DataContext = null;
                    OrderFound = null;
                    OrderFound = osc.GetOrdersforCustomer(CurrOrder.CustomerID);
                    OrderListRO.DataContext = OrderFound;
                    OrderGrid.Visibility = Visibility.Visible;

                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message, "Update Customer Error!", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }

            

        }

        private void OrderListRO_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.Cursor = Cursors.Wait;

            ListView lv = (sender as ListView);
            if (lv.SelectedIndex != -1)
            {
                Order CurrOrder = lv.SelectedItem as Order;
                OrderServiceClient osc = new OrderServiceClient();
                OrderDetailsProductDetailsFound = osc.GetOrderandProductDetailsByOrderID(CurrOrder.StaticDataId);
                OrderDetailsListRO.DataContext = OrderDetailsProductDetailsFound;
                OrderDetailsGrid.Visibility = Visibility.Visible;
            }
            else
            {
                OrderDetailsGrid.Visibility = Visibility.Collapsed;
                OrderDetailsListRO.DataContext = null;
            }

            this.Cursor = Cursors.Arrow;


        }

        private void MenuOrderDetailEdit_Click(object sender, RoutedEventArgs e)
        {
            CurrentOrderDetailProductDetailOrigin = (sender as MenuItem).Tag as OrderDetailProductDetail;
            CurrentOrderDetailProductDetailNew = new OrderDetailProductDetail();
            UpdateOrderDetail(CurrentOrderDetailProductDetailNew, CurrentOrderDetailProductDetailOrigin);

        }

        private void MenuOrderDetailsNewOrder_Click(object sender, RoutedEventArgs e)
        {
            OrderDetailProductDetail CurrOrd = (sender as MenuItem).Tag as OrderDetailProductDetail;
            OrderDetailProductDetail odpd = new OrderDetailProductDetail();
            odpd.OrderID = CurrOrd.OrderID;
            odpd.ProductID = 0;

            NewOrderDetail(odpd);

        }

        private void DeleteOrderDetailMenu_Click(object sender, RoutedEventArgs e)
        {
            MessageBox.Show("DeleteOrderDetailMenu_Click");
        }

        private void OrderDetailsListRO_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

        }

        private void OrderDetailListRO_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            ListView lvi = sender as ListView;
            CurrentOrderDetailProductDetailOrigin = lvi.SelectedItem as OrderDetailProductDetail;

            CurrentOrderDetailProductDetailNew = new OrderDetailProductDetail();
            UpdateOrderDetail(CurrentOrderDetailProductDetailNew, CurrentOrderDetailProductDetailOrigin);

        }

        private void DeleteOrderMenu_Click(object sender, RoutedEventArgs e)
        {
            Order CurrOrder = (sender as MenuItem).Tag as Order;
            if (CurrOrder.StaticDataId != 0)
            {
                try
                {
                    OrderServiceClient osc = new OrderServiceClient();
                    osc.DeleteOrder(CurrOrder.StaticDataId);
                    this.OrderFound = null;
                    this.OrderListRO.DataContext = null;
                    this.OrderFound = osc.GetOrdersforCustomer(CurrOrder.CustomerID);
                    this.OrderListRO.DataContext = this.OrderFound;

                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message, "Delete Order failed");
                }
            }
        }

        private void MenuOrderAddOrderDetail_Click(object sender, RoutedEventArgs e)
        {
            Order CurrOrd = (sender as MenuItem).Tag as Order;
            OrderDetailProductDetail odpd = new OrderDetailProductDetail();
            odpd.OrderID = CurrOrd.StaticDataId;
            odpd.ProductID = 0;

            NewOrderDetail(odpd);


        }




    }
}
