﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using MyServiceAgent.NWOrderService;
using MyServiceAgent.NWProductService;

namespace ETWPFTestClient
{
    /// <summary>
    /// Interaction logic for UpdateOrderDetailWindow.xaml
    /// </summary>
    public partial class UpdateOrderDetailWindow : Window
    {
        private OrderDetailProductDetail CurrentOrderDetails = null;
        private OrderDetailProductDetail OrgOrderDetails = null;
        private Categories[] CategoriesFound = null;
        private Products[] ProductsFound = null;
        private String CurSelDropDown = null;
        private double oldWidth = 0;
        private Int16 NewQuantity = 0;


        public UpdateOrderDetailWindow(OrderDetailProductDetail newOrderDetail)
        {
            InitializeComponent();
            CurrentOrderDetails = newOrderDetail;
            txtQuantity.Focus();
            OrgOrderDetails = new OrderDetailProductDetail();
            CopyOrderDetails(newOrderDetail, OrgOrderDetails);      
            UpdateOrderDetailPanel.DataContext = CurrentOrderDetails;
           

        }
        private void txt_GotFocus(object sender, RoutedEventArgs e)
        {
            TextBox txtGotFocus = (sender as TextBox);
            txtGotFocus.Opacity = 1.0;
            txtGotFocus.FontWeight = FontWeights.Bold;

        }

        private void txt_LostFocus(object sender, RoutedEventArgs e)
        {
            TextBox txtLostFocus = (sender as TextBox);
            txtLostFocus.Opacity = 0.5;
            txtLostFocus.FontWeight = FontWeights.Normal;

        }

        private void bCancelEditOrder_Click(object sender, RoutedEventArgs e)
        {
            UpdateOrderDetailPanel.DataContext = null;
            this.DialogResult = false;
        }

        private void bSaveEditOrder_Click(object sender, RoutedEventArgs e)
        {
            UpdateOrderDetailPanel.DataContext = null;
            this.DialogResult = true;

        }

        private void GetChangeProduct_Click(object sender, RoutedEventArgs e)
        {
            this.Cursor = Cursors.Wait;

            Nullable<Int32> pID = null;
            pID = (sender as Button).Tag as Nullable<Int32>;

            UpdateOrder.IsEnabled = false;
            oldWidth = this.Width;
            this.Width = 1000.0;
            ProductServiceClient psc = new ProductServiceClient();
            CategoriesFound = psc.GetCategories();
            
            CBCategories.DataContext = CategoriesFound.Select( c => c.CategoryName);
            CBCategories.SelectedItem = CurrentOrderDetails.CategoryName;
            lCatDesc.Content = CategoriesFound.Where(c => c.CategoryName == CurrentOrderDetails.CategoryName).First().Description;

            ProductsFound = psc.GetProductsByCategoryName(CurrentOrderDetails.CategoryName);
            ProductListRO.DataContext = ProductsFound;
            ProductListRO.SelectedItem = ProductsFound.Where( c => (c.StaticDataId == CurrentOrderDetails.ProductID)).First();

            txtNewQuantity.Text = CurrentOrderDetails.Quantity.ToString();
            lCurSel.Content = CurrentOrderDetails.ProductName;

            SelectProductGrid.Visibility = Visibility.Visible;

            this.Cursor = Cursors.Arrow;

            
        }

        private void txtDiscount_LostFocus(object sender, RoutedEventArgs e)
        {
            TextBox txtLostFocus = (sender as TextBox);
            CurrentOrderDetails.Discount = Convert.ToSingle( txtLostFocus.Text );

            CurrentOrderDetails.EffectivePrice = CurrentOrderDetails.UnitPrice - (Convert.ToDecimal(CurrentOrderDetails.Discount) * CurrentOrderDetails.UnitPrice);
            CurrentOrderDetails.TotalPrice = CurrentOrderDetails.EffectivePrice * CurrentOrderDetails.Quantity;
            txtLostFocus.Opacity = 0.5;
            txtLostFocus.FontWeight = FontWeights.Normal;


        }

        private void txtQuantity_LostFocus(object sender, RoutedEventArgs e)
        {
            TextBox txtLostFocus = (sender as TextBox);
            CurrentOrderDetails.Quantity = Convert.ToInt16 (txtLostFocus.Text);

            CurrentOrderDetails.TotalPrice = CurrentOrderDetails.EffectivePrice * CurrentOrderDetails.Quantity;
            txtLostFocus.Opacity = 0.5;
            txtLostFocus.FontWeight = FontWeights.Normal;
        }

        private void CBCategories_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.Cursor = Cursors.Wait;

            ComboBox myCombo = (ComboBox)sender;
            CurSelDropDown = myCombo.SelectedItem as String;

            var descs = (from MyServiceAgent.NWProductService.Categories cats in CategoriesFound
                             where cats.CategoryName == CurSelDropDown
                             select cats.Description)  ;

            lCatDesc.Content = descs.ToArray()[0] ;

            ProductServiceClient psc = new ProductServiceClient();
            ProductsFound = psc.GetProductsByCategoryName(CurSelDropDown);
            ProductListRO.DataContext = ProductsFound;

            this.Cursor = Cursors.Arrow;

        }

        private void buttonProductOrder_Click(object sender, RoutedEventArgs e)
        {
            txtDiscount.Focus();
            this.Width = oldWidth;
            Products SelProd =  ProductListRO.SelectedItem as Products;
            CurrentOrderDetails.ProductID = SelProd.StaticDataId;
            CurrentOrderDetails.ProductName = SelProd.ProductName;
            CurrentOrderDetails.Quantity = NewQuantity;
            CurrentOrderDetails.UnitPrice = SelProd.UnitPrice;
            CurrentOrderDetails.CategoryName = CurSelDropDown;

            if (CurrentOrderDetails.Quantity > 0)
            {
                CurrentOrderDetails.EffectivePrice = CurrentOrderDetails.UnitPrice - (Convert.ToDecimal(CurrentOrderDetails.Discount) * CurrentOrderDetails.UnitPrice);
                CurrentOrderDetails.TotalPrice = CurrentOrderDetails.EffectivePrice * CurrentOrderDetails.Quantity;

            }
            CopyOrderDetails(CurrentOrderDetails, OrgOrderDetails );           

            SelectProductGrid.Visibility = Visibility.Collapsed;
            UpdateOrder.IsEnabled = true;

        }

        private void ProductListRO_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListView lv = sender as ListView;
            Products SelProd = null;

            if (ProductListRO.SelectedIndex < 0)
            {
                lCurSelHeader.Visibility = Visibility.Hidden;
                lCurSel.Visibility = Visibility.Hidden;
                buttonProductOrder.IsEnabled = false;
                txtNewQuantity.IsEnabled = false;
                lNoSelHeader.Visibility = Visibility.Visible;
            }
            else
            {
                lNoSelHeader.Visibility = Visibility.Hidden;
                lCurSelHeader.Visibility = Visibility.Visible;
                lCurSel.Visibility = Visibility.Visible;
                buttonProductOrder.IsEnabled = true;
                txtNewQuantity.IsEnabled = true;
                txtNewQuantity.Text = "0";
                SelProd = lv.SelectedItem as Products;
                lCurSel.Content =  SelProd.ProductName;
                NewQuantity = 0;


            } 


        }

        private void ProductListRO_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {

        }

        private void txtNewQuantity_GotFocus(object sender, RoutedEventArgs e)
        {
            TextBox txtGotFocus = (sender as TextBox);
            txtGotFocus.Background = new SolidColorBrush(Colors.LightGreen);
            txtGotFocus.FontWeight = FontWeights.Bold;

        }

        private void txtNewQuantity_LostFocus(object sender, RoutedEventArgs e)
        {
            TextBox txtGotFocus = (sender as TextBox);
            txtGotFocus.Background = new SolidColorBrush(Colors.Transparent);
            txtGotFocus.FontWeight = FontWeights.Normal;
            NewQuantity = Convert.ToInt16(txtGotFocus.Text);

        }

        private void buttonProductCancel_Click(object sender, RoutedEventArgs e)
        {
            txtDiscount.Focus();
            this.Width = oldWidth;
            CopyOrderDetails(OrgOrderDetails, CurrentOrderDetails);           

            SelectProductGrid.Visibility = Visibility.Collapsed;
            UpdateOrder.IsEnabled = true;

        }

        internal void CopyOrderDetails(OrderDetailProductDetail source, OrderDetailProductDetail target)
        {
            target.CategoryName = source.CategoryName;
            target.Description = source.Description;
            target.Discount = source.Discount;
            target.EffectivePrice = source.EffectivePrice;
            target.OrderID = source.OrderID;
            target.ProductName = source.ProductName;
            target.ProductID = source.ProductID;
            target.Quantity = source.Quantity;
            target.QuantityPerUnit = source.QuantityPerUnit;
            target.TotalPrice = source.TotalPrice;
            target.UnitPrice = source.UnitPrice;
            target.UnitsInStock = source.UnitsInStock;
        }




    }
}
